--[[
Bevy Hider
************

A plugin for Bevy o' Bars that creates a small thin bar with one button per Bevy Bar.
Clicking these buttons toggles a bar on/off

Saves only when the UI is unloaded
]]--

import "JackdawPlugins.Bevy.main"
import "JackdawPlugins.Libs.Buttons"


function HiderLoad()
--0) Load data
HiderBar={}
HiderBar.Data=Turbine.PluginData.Load( Turbine.DataScope.Character, "HiderBar")
if HiderBar.Data==nil then
	--Defaults
		HiderBar.Data={}						--Holds all the data
		HiderBar.Data.HideButton={}		--Do we want a button for this bar?
		HiderBar.Data.Hidden={}			--Do we want this bar to be hidden?
	for i=1,NumberOfBars do
		--Default> Create a hider Button for every Bar
		HiderBar.Data.HideButton[i]=true
		HiderBar.Data.Hidden[i]=false
	end
		--Default to top left corner
		HiderBar.Data.x=0
		HiderBar.Data.y=0
end
end

function BuildHiderBar()
--Default to having a hide bar button.
for i=1,NumberOfBars do
	if HiderBar.Data.HideButton[i]==nil then HiderBar.Data.HideButton[i]=true end
end
--1) create window to hold buttons
HiderBar.Window=JackdawPlugins.Libs.NewWindow{x=HiderBar.Data.x,y=HiderBar.Data.y, height=10, width=20+10*sum(HiderBar.Data.HideButton),colour=Turbine.UI.Color(0.8,0,0,0.3), name="Hider Bar"}
HiderBar.DragBar = Deusdictum.UI.DragBar( HiderBar.Window, "Hider Bar")

--2) create a button for each bar
HiderBar.Button={}
local j=0
for i=1,NumberOfBars do
	if HiderBar.Data.HideButton[i] then
		j=j+1
		HiderBar.Button[i] = JackdawPlugins.Libs.NewButton{x=10*j,y=1,width=8,height=8,parent=HiderBar.Window,colour=Turbine.UI.Color(1,BBar[i].options.colour.R,BBar[i].options.colour.G,BBar[i].options.colour.B) or Turbine.UI.Color(1,1,0,1)}
		--if BBar[i].Window:IsVisible()
		if HiderBar.Data.Hidden[i]==true then
			BBar[i].Window:SetVisible(false)
			HiderBar.Button[i]:SetText ("X")
			else
			BBar[i].Window:SetVisible(true)
			HiderBar.Button[i]:SetText ("O")
		end
		HiderBar.Button[i]:SetOutlineColor( Turbine.UI.Color( 1,0, 0, 0 ) )
        HiderBar.Button[i]:SetFontStyle( Turbine.UI.FontStyle.Outline )
        HiderBar.Button[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
		HiderBar.Button[i]:SetFont(Turbine.UI.Lotro.Verdana10)
			
		HiderBar.Button[i].MouseClick=function(sender,args)
		
		--Save on unload
		--when Turbine finally fix that bug - move this somewhere more sensible!
		if hset==nil then
			pcall(function () 
				if ( Plugins ["BevyHider"] ~= nil ) then
					JackdawPlugins.Libs.echo("Bevy Hider registered for save on unload")
					hset=true
					Plugins["BevyHider"].Unload = function( sender, args )
						HiderBar.Data.x,HiderBar.Data.y=HiderBar.Window:GetPosition()
						Turbine.PluginData.Save( Turbine.DataScope.Character, "HiderBar", HiderBar.Data );
						JackdawPlugins.Libs.echo("Thank you for using Bevy Hider!")
					end
				end
			end)
		end
		
		
		
		--Show or hide windows...
			if BBar[i].Window:IsVisible() then
				BBar[i].Window:SetVisible(false)
				HiderBar.Button[i]:SetText ("X")
				HiderBar.Data.Hidden[i]=true
			else
				BBar[i].Window:SetVisible(true)
				HiderBar.Button[i]:SetText ("O")
				HiderBar.Data.Hidden[i]=false
			end
		end
	end
end

--4)Create a button to show options
	HiderBar.OptionsButton=JackdawPlugins.Libs.NewButton{x=1,y=1,width=8,height=8,parent=HiderBar.Window,colour=Turbine.UI.Color(1,1,0,0)}
	HiderBar.OptionsButton:SetVisible(false)
	HiderBarContextEvent()

--5)Window events
--HiderBar.Window:SetWantsUpdates(true); I do not think this line is needed...
HiderBar.Window:SetWantsKeyEvents(true);

HiderBar.Window.KeyDown = function(sender, args)
	if (args.Action == 268435579) then
		--reveal or hide button
		HiderBar.OptionsButton:SetVisible(not HiderBar.OptionsButton:IsVisible())
	end
end
end

function sum (t)
	local foo=0
	for i=1, #t do
		if t[i]==true then
			foo=foo+1
		end
	end
	return foo
end

function HiderBarContextEvent()
--The contextmenu for the hiderbar button!
	HiderBar.OptionsButton.MouseDown = function( sender, args )
	-- build the menu items in the context menu
		local contextMenu = Turbine.UI.ContextMenu();
		local contextMenuItems = contextMenu:GetItems();

		local ButtonFor = Turbine.UI.MenuItem( "Button For..." )
		contextMenu.ButtonForSubmenuItems= ButtonFor:GetItems()
			HideBar={}
			for i=1,NumberOfBars do
				HideBar[i] = Turbine.UI.MenuItem( "Bar "..i,true,HiderBar.Data.HideButton[i] );
				HideBar[i].Click = function( sender, args )
					HiderBar.Data.HideButton[i]=not(HiderBar.Data.HideButton[i])
					--Now rebuild the bar
					BuildHiderBar()
					--Tell the UI we are unlocked i.e. show the dragbar and options button
					HiderBar.DragBar:ToggleDraggable()
					HiderBar.OptionsButton:SetVisible()
				end
				contextMenu.ButtonForSubmenuItems:Add(HideBar[i])
			end
		contextMenuItems:Add( ButtonFor )
		contextMenu:ShowMenu();
	end
end

--Let's go!
HiderLoad()
BuildHiderBar()