import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Utils";
import "Turbine.UI.Extensions";
import "KragenPlugs.KragenBars";

Bar = class (Turbine.UI.Extensions.SimpleWindow)

function Bar:Constructor (exclusive)

	Turbine.UI.Extensions.SimpleWindow.Constructor( self );
	
	-- Variables for storing and tracking instances of Slot Class
	self.totalslots = 0;
	self.slot = { };
	
	-- Timer for Early Warnings/Skill Hacks
	self.timer = Timer();
	self.timerOn = true;
	self.timerEnd = true;
	
	-- Effect ID codes
	self.effectID = 0;
	self.timerID = 0;
	
	-- Multi-Bar Reactive Chain.
	-- Exclusive means it only shows the highest part of the chain that is active.
	self.exclusive = exclusive;
	
	-- Do you want to fade down to partial opacity when not hovering over bar
	self.mousehover = false;
	
	-- When Timer Elapses 
	-- make the bar visible at half opacity (true - ie Ward)
	-- or set to invisble and reset EffectID (false - ie Burglar's Advantage)
	self.timer.TimerElapsed = function ()
	    if (self.timerEnd) then
			self:SetOpacity( settings.opacity *.5 );
			self:SetVisible( true );
		else
		    self.effectID = 0;
		    self:SetVisible( false );
		end
	end
	
	-- if hover effect is on, set to full opacity when mouse enters bar	
	self.MouseEnter = function (sender, args)
		if (self.mousehover) then
			self:SetOpacity(settings.opacity);
		end
	end
	
	-- if hover effect is on, set to half opacity when mouse leaves bar
	self.MouseLeave = function (sender, args)
		if (self.mousehover) then
			self:SetOpacity(settings.opacity * .5);
		end
	end
end

-- Add quickslot to bar
function Bar:AddQuickSlot(id, posx, posy, hex)
	self.totalslots = self.totalslots + 1;
	
	self.slot[id] = Slot();
	self.slot[id]:SetParent( self );
	self.slot[id]:SetSize( 35, 35 );
	self.slot[id]:SetPosition( ( posx - 1 ) * ( 35 ), ( posy - 1 ) * 35 );
	if (hex ~= "") then
		self.slot[id]:SetShortcut( Turbine.UI.Lotro.Shortcut( 6.000000, hex ) );
	end
	self.slot[id]:SetAllowDrop(false);

end

-- Start the timer
function Bar:StartTimer(duration, bol)
	self.timer:Stop();
	self.timer:SetInterval(duration);
	self.timer:Start();
	self.timerEnd = bol;
end

-- Set the mouse hover
function Bar:SetHover(bol)
	self.mousehover = bol;
	if (bol) then
		self:SetOpacity(settings.opacity * .3);
	else
	    self:SetOpacity(settings.opacity);
	end
end