import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "Turbine.Utils";
import "KragenPlugs.KragenBars";

ChampBars = class( );

function ChampBars:Constructor()

    -- inialize the setupWindow
	self.setup = SetupWindow();

	-- class settings arnt saved then load the default settings
	if (not settings.class) then
	    settings.class = "Champion";
	    self:DefaultSettings();
	    self:ShowSetup();
	else
		self.settings = settings.classinfo;
	end

    self.window = { };

	-- Defeat Response Generic
	self.window[1] = Window("DEFEAT1", 3, 1, -1.5, 1, self.settings.defeat1);
    
		-- Generic Defeat Responses
		self.window[1]:AddBar(1, 3, 1, 0, 0, false, false);
		self.window[1].bar[1]:AddQuickSlot(1, 1, 1, "0x7000D47A"); -- Second Wind
		self.window[1].bar[1]:AddQuickSlot(2, 2, 1, "0x7000967D"); -- Red Haze
		self.window[1].bar[1]:AddQuickSlot(3, 3, 1, "0x70020DF3"); -- Heroics
		
    -- Defeat Response Generic
	self.window[2] = Window("DEFEAT2", 3, 1, -1.5, 0, self.settings.defeat2);
		
		-- Blocking Blades
		self.window[2]:AddBar(1, 1, 1, 0, 0, false, false);
		self.window[2].bar[1]:AddQuickSlot(1, 1, 1, "0x70009679");
		
		-- Flurry
		self.window[2]:AddBar(2, 1, 1, 1, 0, false, false);
		self.window[2].bar[2]:AddQuickSlot(1, 1, 1, "0x70003E00");
		self.window[2].bar[2]:SetZOrder(0);

		-- Exchange
		self.window[2]:AddBar(3, 1, 1, 2, 0, false, false);
		self.window[2].bar[3]:AddQuickSlot(1, 1, 1, "0x70003E02");
		self.window[2].bar[3]:SetZOrder(0);
		
	-- Buffs
	self.window[3] = Window("BUFFS", 2, 1, -.5, 0, self.settings.buffs);
	self.window[3]:SetZOrder( 5 );
    
		-- Flurry
		self.window[3]:AddBar(1, 1, 1, 0, 0, self.settings.buffs.flurry, false);
		self.window[3].bar[1]:AddQuickSlot(1, 1, 1, "0x70003E00");
		self.window[3].bar[1]:SetZOrder(5);
		
		-- Exchange
		self.window[3]:AddBar(2, 1, 1, 1, 0, self.settings.buffs.exchange, false);
		self.window[3].bar[2]:AddQuickSlot(1, 1, 1, "0x70003E02");
		self.window[3].bar[2]:SetZOrder(5);
		
		
    -- Disarm/Stun Indicator
    self.window[4] = Window("EFFECT", 1, 1, 2.5, 0, self.settings.effect);
    
		-- Blood Rage Indicator
		self.window[4]:AddBar(1, 1, 1, 0, 0, false, false);
		self.window[4].bar[1]:AddQuickSlot(1, 1, 1, "0x7000F70E"); -- Blood-Rage
		self.window[4].bar[1]:SetZOrder(0);
		
		-- Hedge Indicator
		self.window[4]:AddBar(2, 1, 1, 0, 0, false, false);
		self.window[4].bar[2]:AddQuickSlot(1, 1, 1, "0x70003E0F"); -- Hedge
		self.window[4].bar[2]:SetZOrder(5);
		
	
 	-- STANCE WINDOW
	self.window[5] = Window("STANCE", 1, 1, 3.5, 1, self.settings.stance);


		-- Stance Indicator
		self.window[5]:AddBar(1, 1, 1, 0, 0, true, false);
		self.window[5].bar[1]:AddQuickSlot(1, 1, 1, ""); -- PrimaryStance
        self.window[5].bar[1].slot[1]:MultiShortcut(6, "0x70003DFF"); -- Fervour
		self.window[5].bar[1].slot[1]:MultiShortcut(6, "0x7000D46C"); -- Ardour
		self.window[5].bar[1].slot[1]:MultiShortcut(6, "0x7000967C"); -- Glory
		self.window[5].bar[1].slot[1]:MultiShortcut(6, "0x70002DB5"); -- Controlled Burn
		self.window[5].bar[1].slot[1]:SetScroll(true);
		self.window[5].bar[1].slot[1]:SetMulti(self.settings.stance.primary);
		if (self.settings.stance.controlled) then
			self.window[5].bar[1].slot[1]:ChangeTotal(4);
		else
	    	self.window[5].bar[1].slot[1]:ChangeTotal(3);
		end
		self.settings.stance.current = 0;
		
		
	-- SET WINDOW OPACITY
	self.window[1]:SetOpacity(settings.opacity);
	self.window[2]:SetOpacity(settings.opacity);
	self.window[3]:SetOpacity(settings.opacity);
	self.window[4]:SetOpacity(settings.opacity);
	self.window[5]:SetOpacity(settings.opacity);
	
	
	-- SET WINDOW VISIBILITY
	self.window[1]:SetVisible(self.settings.defeat1.active);
	self.window[2]:SetVisible(self.settings.defeat2.active);
	self.window[3]:SetVisible(self.settings.buffs.active);
	self.window[4]:SetVisible(self.settings.effect.active);
	self.window[5]:SetVisible(self.settings.stance.active);
		
	
	-- Hook into the player to receive updates about player morale and
	-- power.
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
	else
		self.player = Turbine.Gameplay.LocalPlayer();
	end

	local effects = self.player:GetEffects();

	effects.EffectAdded = function( sender, args )
		self:AddEffect( args.Index );
	end

	effects.EffectRemoved = function( sender, args )
		self:RemoveEffect( args.Effect );
	end
end


function ChampBars:AddEffect( effectIndex )
	local addedeffect = self.player:GetEffects():Get( effectIndex );

	-- Defeat Response
	if (addedeffect:GetName() == "Enemy Defeat Response") then
        self.window[1]:BarActive(1 , addedeffect:GetID(), true, true);
        if (self.window[2].bar[1].effectID == 0 and self.settings.stance.current ~= 1) then
			self.window[2].bar[1]:SetVisible(true);
		end
        if (self.window[2].bar[2].effectID == 0 and (self.settings.stance.current == 2 or self.settings.stance.current == 4)) then
			self.window[2].bar[2]:SetVisible(true);
			self.window[2]:SetZOrder( 10 );
		end
	    if (self.window[2].bar[3].effectID == 0 and (self.settings.stance.current == 3 or self.settings.stance.current == 4)) then
			self.window[2].bar[3]:SetVisible(true);
			self.window[2]:SetZOrder( 10 );
		end
        
    -- Blocking Blades
	elseif (addedeffect:GetName() == "Blocking Blades") then
        self.window[2]:BarActive(1 , addedeffect:GetID(), true, false);

	-- Ardent Flurry
	elseif (addedeffect:GetName() == "Ardent Flurry") then
  		self.window[2]:BarActive(2 , addedeffect:GetID(), true, false);

    -- Glorious Exchange
	elseif (addedeffect:GetName() == "Glorious Exchange") then
        self.window[2]:BarActive(3 , addedeffect:GetID(), true, false);
        
	-- Flurry
	elseif (addedeffect:GetName() == "Flurry" or addedeffect:GetName() == "Improved Flurry") then
	    self.window[3]:BarActive(1 , addedeffect:GetID(), true, false);
	    if (self.settings.buffs.timer) then
			self.window[3].bar[1]:StartTimer(addedeffect:GetDuration() * .70, true);
		end
		
    -- Exchange
 	elseif (addedeffect:GetName() == "Exchange of Blows" or addedeffect:GetName() == "Improved Exchange of Blows") then
	    self.window[3]:BarActive(2 , addedeffect:GetID(), true, false);
	    if (self.settings.buffs.timer) then
			self.window[3].bar[2]:StartTimer(addedeffect:GetDuration() * .70, true);
		end
		
    -- Stunned, Dazed, Knocked Down
	elseif (addedeffect:GetName() == "Stunned" or addedeffect:GetName() == "Dazed" or addedeffect:GetName() == "Knocked Down") then
  		self.window[4]:BarActive(1 , addedeffect:GetID(), true, true);
  		self.window[4].bar[1]:SetZOrder(10);

	-- Disarmed
	elseif (addedeffect:GetName() == "Primary Weapon: Disarmed") then
  		self.window[4]:BarActive(2 , addedeffect:GetID(), true, true);

   -- Stance Change
	elseif (addedeffect:GetName() == "Champion's Fervour" or addedeffect:GetName() == "Ardour" or addedeffect:GetName() == "Glory" or addedeffect:GetName() == "Controlled Burn") then
  		self.window[5]:BarActive(1, addedeffect:GetID(), true, true);
		self.window[5].bar[1]:SetHover( true );

		if (addedeffect:GetName() == "Champion's Fervour") then
			self.settings.stance.current = 1;
			if (self.window[3].bar[1].effectID == 0) then
				self.window[3].bar[1]:SetVisible( true );
			end
 			if (self.window[3].bar[2].effectID == 0) then
				self.window[3].bar[2]:SetVisible( true );
			end

		elseif (addedeffect:GetName() == "Ardour") then
			self.settings.stance.current = 2;
			self.window[3].bar[1]:SetVisible( false );
 			if (self.window[3].bar[2].effectID == 0) then
				self.window[3].bar[2]:SetVisible( true );
				self.window[3].bar[2]:SetOpacity( settings.opacity );
			end
		elseif (addedeffect:GetName() == "Glory") then
			self.settings.stance.current = 3;
			self.window[3].bar[2]:SetVisible( false );
			if (self.window[3].bar[1].effectID == 0) then
				self.window[3].bar[1]:SetVisible( true );
				self.window[3].bar[1]:SetOpacity( settings.opacity );
			end
		elseif (addedeffect:GetName() == "Controlled Burn") then
			self.settings.stance.current = 4;
			self.window[3].bar[1].timer:Stop();
			self.window[3].bar[2].timer:Stop();
			self.window[3].bar[1]:SetVisible( false );
			self.window[3].bar[2]:SetVisible( false );
		end

		if (self.settings.stance.current ~= self.settings.stance.primary) then
			self.window[5].bar[1].slot[1]:SetMulti(self.settings.stance.primary);
		else
			self.window[5].bar[1].slot[1]:SetMulti(self.settings.stance.secondary);
		end

    end
end

function ChampBars:RemoveEffect( effect )

	-- Defeat Response
	if (effect:GetName() == "Enemy Defeat Response") then
  		self.window[1]:BarActive(1 , effect:GetID(), false, false);
		if (self.window[1].bar[1].effectID == 0) then
			self.window[2].bar[1]:SetVisible(false);
			self.window[2].bar[2]:SetVisible(false);
			self.window[2].bar[3]:SetVisible(false);
			self.window[2]:SetZOrder( 0 );
		end

	-- Blocking Blades
    elseif (effect:GetName() == "Blocking Blades") then
        self.window[2]:BarActive(1 , effect:GetID(), false, false);
        if (self.window[1].bar[1].effectID ~= 0 and self.window[2].bar[1].effectID == 0 and self.settings.stance.current ~= 1) then
			self.window[2].bar[1]:SetVisible(true);
		end
        
    -- Ardent Flurry
    elseif (effect:GetName() == "Ardent Flurry") then
     	self.window[2]:BarActive(2 , effect:GetID(), false, false);
     	if (self.window[1].bar[1].effectID ~= 0 and self.window[2].bar[2].effectID == 0 and (self.settings.stance.current == 2 or self.stance.current == 4)) then
			self.window[2].bar[2]:SetVisible(true);
		end

	-- Glorious Exchange
    elseif (effect:GetName() == "Glorious Exchange") then
        self.window[2]:BarActive(3 , effect:GetID(), false, false);
	    if (self.window[1].bar[1].effectID ~= 0 and self.window[2].bar[3].effectID == 0 and (self.settings.stance.current == 3 or self.stance.current == 4)) then
			self.window[2].bar[3]:SetVisible(true);
		end
		
    -- Flurry
	elseif (effect:GetName() == "Flurry" or effect:GetName() == "Improved Flurry") then
        self.window[3]:BarActive(1 , effect:GetID(), false, false);
        if (self.settings.buffs.flurry and self.window[3].bar[1].effectID == 0 and (self.settings.stance.current ~= 2 and self.settings.stance.current ~= 4)) then
			self.window[3].bar[1]:SetVisible( true );
			self.window[3].bar[1]:SetOpacity( settings.opacity );
		end


	-- Exchange
	elseif (effect:GetName() == "Exchange of Blows" or effect:GetName() == "Improved Exchange of Blows") then
        self.window[3]:BarActive(2 , effect:GetID(), false, false);
        if (self.settings.buffs.exchange and self.window[3].bar[2].effectID == 0 and (self.settings.stance.current ~= 3 and self.settings.stance.current ~= 4)) then
			self.window[3].bar[2]:SetVisible( true );
			self.window[3].bar[2]:SetOpacity( settings.opacity );
		end

    -- Stunned, Dazed or Knocked Down
	elseif (effect:GetName() == "Stunned" or effect:GetName() == "Dazed" or effect:GetName() == "Knocked Down") then
  		self.window[4]:BarActive(1 , effect:GetID(), false, false);
		if (self.window[4].bar[1].effectID == 0) then
		    self.window[4].bar[1]:SetZOrder(0);
		end

	-- Disarmed
	elseif (effect:GetName() == "Primary Weapon: Disarmed") then
        self.window[4]:BarActive(2 , effect:GetID(), false, false);

	-- Stance Change
 	elseif (effect:GetName() == "Champion's Fervour" or effect:GetName() == "Ardour" or effect:GetName() == "Glory" or effect:GetName() == "Controlled Burn") then
     	self.window[5]:BarActive(1, effect:GetID(), false, true);
    	if (self.window[5].bar[1].effectID == 0) then
			self.window[5].bar[1]:SetHover( false );
			self.window[5].bar[1].slot[1]:SetMulti(self.settings.stance.primary);
			self.settings.stance.current = 0;
			
			if (self.window[3].bar[1].effectID == 0) then
				self.window[3].bar[1]:SetVisible( true );
			end
 			if (self.window[3].bar[2].effectID == 0) then
				self.window[3].bar[2]:SetVisible( true );
			end
		end


	end
end

function ChampBars:DefaultSettings()
	self.settings = { };

	self.settings.defeat1 = { };
	self.settings.defeat1.active = true;
	self.settings.defeat1.x = 0;
	self.settings.defeat1.y = 0;

	self.settings.defeat2 = { };
	self.settings.defeat2.active = true;
	self.settings.defeat2.x = 0;
	self.settings.defeat2.y = 0;

	self.settings.buffs = { };
	self.settings.buffs.active = true;
	self.settings.buffs.x = 0;
	self.settings.buffs.y = 0;
	self.settings.buffs.timer = true;
	self.settings.buffs.flurry = true;
	self.settings.buffs.exchange = true;

	self.settings.effect = { };
	self.settings.effect.active = true;
	self.settings.effect.x = 0;
	self.settings.effect.y = 0;

	self.settings.stance = { };
	self.settings.stance.active = true;
	self.settings.stance.x = 0;
	self.settings.stance.y = 0;
	self.settings.stance.current = 1;
	self.settings.stance.primary = 1;
	self.settings.stance.secondary = 3;
	self.settings.stance.controlled = true;
end

function ChampBars:SaveSettings()
	self.settings.defeat1.x, self.settings.defeat1.y = self.window[1]:GetPosition();
	self.settings.defeat2.x, self.settings.defeat2.y = self.window[2]:GetPosition();
	self.settings.buffs.x, self.settings.buffs.y = self.window[3]:GetPosition();
	self.settings.effect.x, self.settings.effect.y = self.window[4]:GetPosition();
	self.settings.stance.x, self.settings.stance.y = self.window[5]:GetPosition();
	settings.classinfo = self.settings;
	SaveSettings();
end

function ChampBars:ShowSetup()
    self.setup:SetVisible(true);
	self.setup.slider1:SetValue(settings.opacity);
	
	self.setup.checkbox1:SetText("Defeat Response 1");
	self.setup.checkbox2:SetText("Defeat Response 2");
	self.setup.checkbox3:SetText("Buffs");
  	self.setup.checkbox4:SetText("Stun/Disarm Effect");
    self.setup.checkbox5:SetText("Stance Slot");
    self.setup.checkbox6:SetText("Controlled Burn");
    self.setup.checkbox7:SetText("Flurry");
    self.setup.checkbox8:SetText("Exchange of Blows");
    self.setup.checkbox9:SetVisible( false );
    self.setup.checkbox10:SetVisible( false );
    self.setup.checkbox11:SetText("Early Warning");

  	self.setup.checkbox1:SetChecked( self.settings.defeat1.active );
  	self.setup.checkbox2:SetChecked( self.settings.defeat2.active );
  	self.setup.checkbox3:SetChecked( self.settings.buffs.active );
  	self.setup.checkbox4:SetChecked( self.settings.effect.active );
  	self.setup.checkbox5:SetChecked( self.settings.stance.active );
  	self.setup.checkbox6:SetChecked( self.settings.stance.controlled );
  	self.setup.checkbox7:SetChecked( self.settings.buffs.flurry );
  	self.setup.checkbox8:SetChecked( self.settings.buffs.exchange );
  	self.setup.checkbox11:SetChecked( self.settings.buffs.timer );
  	
    il={"Fervour", "Ardour", "Glory", "Controlled Burn"};
	self.setup.dropdown1 = KragenPlugs.DUInterface.DropDownBox(il);
	self.setup.dropdown1:SetParent(self.setup.bg);
	self.setup.dropdown1:SetPosition(20,305);
	self.setup.dropdown1:SetZOrder(99); -- can be less, but should be higher than others to allow the drop down to cover other elements
  	il={"Fervour", "Ardour", "Glory", "Controlled Burn"};
	self.setup.dropdown2 = KragenPlugs.DUInterface.DropDownBox(il);
	self.setup.dropdown2:SetParent(self.setup.bg);
	self.setup.dropdown2:SetPosition(215,305);
	self.setup.dropdown2:SetZOrder(99); -- can be less, but should be higher than others to allow the drop down to cover other elements

	self.setup.dropdown1.SetIndex(self.settings.stance.primary);
    self.setup.dropdown2.SetIndex(self.settings.stance.secondary);
	self.setup:SetText("KB: Champion");
end

function ChampBars:SetupClosing()
    self.settings.defeat1.active = self.setup.checkbox1:GetChecked();
	self.settings.defeat2.active = self.setup.checkbox2:GetChecked();
	self.settings.buffs.active = self.setup.checkbox3:GetChecked();
	self.settings.effect.active = self.setup.checkbox4:GetChecked();
	self.settings.stance.active = self.setup.checkbox5:GetChecked();
	self.settings.stance.primary = self.setup.dropdown1:GetIndex();
	self.settings.stance.secondary = self.setup.dropdown2:GetIndex();
	self.settings.stance.controlled = self.setup.checkbox6:GetChecked();
	self.settings.buffs.flurry = self.setup.checkbox7:GetChecked();
	self.settings.buffs.exchange = self.setup.checkbox8:GetChecked();
	self.settings.buffs.timer = self.setup.checkbox11:GetChecked();
	
	if (self.settings.stance.controlled) then
		self.window[5].bar[1].slot[1]:ChangeTotal(4);
	else
	   	self.window[5].bar[1].slot[1]:ChangeTotal(3);
	end
	
	self.window[3].bar[1]:SetVisible(self.settings.buffs.flurry and self.window[3].bar[1].effectID == 0 and (self.settings.stance.current ~= 2 and self.settings.stance.current ~= 4));
	self.window[3].bar[2]:SetVisible(self.settings.buffs.exchange and self.window[3].bar[2].effectID == 0 and (self.settings.stance.current ~= 3 and self.settings.stance.current ~= 4));
	self.window[3].bar[1]:SetOpacity( settings.opacity );
	self.window[3].bar[2]:SetOpacity( settings.opacity );
	
	self.window[1].display(self.settings.defeat1.active);
	self.window[2].display(self.settings.defeat2.active);
	self.window[3].display(self.settings.buffs.active);
	self.window[4].display(self.settings.effect.active);
	self.window[5].display(self.settings.stance.active);

	-- SET WINDOW OPACITY
	self.window[1]:SetOpacity(settings.opacity);
	self.window[2]:SetOpacity(settings.opacity);
	self.window[3]:SetOpacity(settings.opacity);
	self.window[4]:SetOpacity(settings.opacity);
	self.window[5]:SetOpacity(settings.opacity);

	self:SaveSettings();
end