import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Utils";
import "KragenPlugs.KragenBars";

-- load settings file
function LoadSettings()
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "KragenBarsSettings" );

	-- declare settings as a table and set default opacity
	if ( type( settings ) ~= "table" ) then
		settings = { };
		settings.opacity = 1;
	end
	
	-- if version is not declared then wipe settings file and set version number
	if (settings.vers == nil) then
        settings = nil;
	    settings = { };
	    settings.vers = "1.5.0";
	    settings.opacity = 1;
	    SaveSettings();
	end
	
	-- if classbars are already setup
	if ( settings.class == "Guardian") then
		KragenBars = GuardBars();
	elseif (settings.class == "Burglar") then
	    KragenBars = BurgBars();
 	elseif (settings.class == "Champion") then
	    KragenBars = ChampBars();
	elseif (settings.class == "Captain") then
		KragenBars = CaptBars();
	else
	    -- inform the user how to setup the classbar
		Turbine.Shell.WriteLine(" Welcome to KragenBars\n - To set up your class specific bars type /kragenbars <guardian | burglar | champion | captain>");
	end

end

-- save to settings file
function SaveSettings()
	
	Turbine.PluginData.Save( Turbine.DataScope.Character, "KragenBarsSettings", settings );

end

-- on unload call each instances unload function to save settings
function UnloadPlugin()
	KragenBars:SaveSettings();
end


-- setup user chat commands
KragenBarsCommand = Turbine.ShellCommand();

function KragenBarsCommand:Execute( command, arguments )
	-- setup command
	if ( arguments == "setup" ) then
		-- if class bar is already setup
		if (settings.class) then
		   KragenBars:ShowSetup();
		   
		else
		    -- inform the user how to setup the classbar
      		Turbine.Shell.WriteLine(" You Must set up your class specific bars first\n - To set up your class specific bars type /kragenbars <guardian | burglar | champion | captain>");
		end
	    
	else
   		
		-- commands for settup up class bar
  		if (not settings.class) then
			if ( arguments == "guardian" ) then
				KragenBars = GuardBars();
				Turbine.Shell.WriteLine(" To recall the setup window in the future type /kragenbars setup");
			elseif ( arguments == "burglar" ) then
				KragenBars = BurgBars();
				Turbine.Shell.WriteLine(" To recall the setup window in the future type /kragenbars setup");
			elseif (arguments == "captain" ) then
				KragenBars = CaptBars();
				Turbine.Shell.WriteLine(" To recall the setup window in the future type /kragenbars setup");
			elseif ( arguments == "champion" ) then
				KragenBars = ChampBars();
				Turbine.Shell.WriteLine(" To recall the setup window in the future type /kragenbars setup");
			else
       			-- inform the user how to setup the classbar
      			Turbine.Shell.WriteLine(" You Must set up your class specific bars first\n - To set up your class specific bars type /kragenbars <guardian | burglar | champion | captain>");
			end
		else
            Turbine.Shell.WriteLine(" To access the setup window type /kragenbars setup");
		end
	end

end

-- inialize plugin
function init()
    settings = { }

	-- setup plugin unload function to save setting on unload.
		pluginName="KragenBars";
		window = Turbine.UI.Window();
		window:SetWantsUpdates(true);
		window.Update = function(sender,args)
			if (Plugins[pluginName] ~= nil) then
				Plugins[pluginName].Unload = function(self,sender,args)
					UnloadPlugin();
				end
				window:SetWantsUpdates(false);
			end
		end

	Turbine.Shell.AddCommand( "kragenbars", KragenBarsCommand );

	LoadSettings();
end


-- codeblock
init();