import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Utils";
import "KragenPlugs.KragenBars";
import "Deusdictum.UI.DragBar";

Window = class (Turbine.UI.Window)

function Window:Constructor (heading, width, height, posx, posy, settings)
	Turbine.UI.Window.Constructor( self );
	
	self.declared = false;
	
	self.settings = settings;
	-- variables for tracking and storing instances of Bar class
	self.totalbars = 0;
	self.bar = { };
	
	-- display width/height for default positioning
	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();
	
	-- sizing and positioning
	self:SetSize((width * 35), (height * 35));
	if (self.settings.x == 0 or self.settings.y == 0) then
		self:SetPosition((displayWidth / 2) + (posx * 35) , (displayHeight * 2 / 3) + (posy * 35));
	else
		self:SetPosition(self.settings.x, self.settings.y);
	end
		
	-- save position when user stops dragging
	self.DragEnd = function(sender)
        local x, y = self:GetPosition();
        self.x = x;
        self.y = y;
    end
    
    self.HUDShow = function(sender)
    	if (self.declared) then
    		self:SetVisible(self.settings.active);    	
    	end
    	self.declared = true;
	end
	
	self.display = function(bol)
		self.settings.active = bol;
		self:SetVisible(bol);
	end
	
	-- attach dragbar class to window
  	self.dragBar = Deusdictum.UI.DragBar( self, heading );
end

-- Add bar to window
function Window:AddBar(id, width, height, posx, posy, visible, exclusive)
	if (not self.bar[id]) then
		self.totalbars = self.totalbars + 1;
    	self.bar[id] = Bar(exclusive);
		self.bar[id]:SetParent( self );
		self.bar[id]:SetSize((width * 35), (height * 35));
		self.bar[id]:SetPosition( posx * 35, posy * 35 );
		self.bar[id]:SetVisible( visible );
	end
end

-- Change active bar/effect state
function Window:BarActive(barID, effectID, effect, visible)
	-- change effect states
	if (effect) then
		self.bar[barID].effectID = effectID;	
	else
		if (self.bar[barID].effectID == effectID or effectID == 0) then
			self.bar[barID].effectID = 0;
		end
	end
	
	-- change visibility state
	if (visible) then
		local laterstage = false;
		
		-- if bar state is exclusive then check if any later bars are active already
		if (self.bar[barID].exclusive) then
			for i = (barID + 1), (self.totalbars) do
				if (self.bar[i].exclusive and self.bar[i].effectID ~= 0) then
					laterstage = true;
				end
			end
			for i = 1, self.totalbars do
				if (i < barID and self.bar[i].exclusive) then
					self.bar[i]:SetVisible( false );
				end
			end
		end
		
		-- display bar only if later bars are not active
		if (not laterstage) then
		    if (not self.bar[barID]:IsVisible() and (self.bar[barID].effectID == 0 or effect)) then
				self.bar[barID]:SetVisible( true );
			end
			if (not self.bar[barID].mousehover and (self.bar[barID].effectID == 0 or effect)) then
				self.bar[barID]:SetOpacity( settings.opacity );
			end
		end
		
	else
		if (self.bar[barID].effectID == 0 or effect) then
			self.bar[barID]:SetVisible( false );
		end
		
		local earlystage = 0;
		
		-- if bar state is exclusive then check if any earlier stages are active
		if (self.bar[barID].exclusive) then
			for i = 1, barID do
				if (self.bar[i].exclusive and self.bar[i].effectID ~= 0) then
					earlystage = i;
				end
			end
		end
		
		-- display the highest stage earlier than the current stage
		if (earlystage ~= 0 and earlystage ~= barID) then
			self.bar[earlystage]:SetVisible( true );
			if (not self.bar[earlystage].mousehover) then
				self.bar[earlystage]:SetOpacity( settings.opacity );
			end
		end
		
	end
end

-- Change opacity of window
function Window:ChangeOpacity(opacity)
	for c = 1, self.totalbars do
		self.bar[c]:SetOpacity(opacity);
	end
end