import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

EffectSettings = class();

function EffectSettings:Constructor(saved)
    if (saved ~= nil) then
        for i,v in pairs(saved) do
            self[i] = v;
        end
    else
        self:SetDefaults();
    end
end

function EffectSettings:Copy()
    return PengorosPlugins.Utils.DeepCopy(self);
end

function EffectSettings:SetDefaults()
    -- effect bars
    self.enabled = true;
    self.name = "BuffBars";
    self.posX = 10;
    self.posY = 200;
    self.barHeight = 24;
    self.barWidth = 200;

    -- effect category filters
    self.showDisease = true;
    self.showFear = true;
    self.showPoison = true;
    self.showWound= true;
    self.showCorruption = true;
    self.showCry = true;
    self.showElemental = true;
    self.showPhysical = true;
    self.showSong = true;
    self.showUndefined = true;
    
    -- cooldown category filters
    self.showDiseaseCD = true; -- v4
    self.showFearCD = true; -- v4
    self.showPoisonCD = true; -- v4
    self.showWoundCD = true; -- v4
    self.showMoraleCD = true; -- v4
    self.showPowerCD = true; -- v4

    -- effect type filters
    self.showCurableOnly = false;
    self.ignored = {};
    self.override = {};
    
    -- effect duration filters
    self.minDuration = 0; -- v4
    self.maxDuration = 0; -- v4
end

function EffectSettings:GetColor(color)
    return Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255);
end

function EffectSettings:ResetIgnored()
    self.ignored = {};
end

function EffectSettings:AddIgnoredEffect(effect)
    self.ignored[effect] = true;
end

function EffectSettings:RemoveIgnoredEffect(effect)
    self.ignored[effect] = nil;
end

function EffectSettings:IsIgnoredEffect(effect)
    return self.ignored[effect];
end

function EffectSettings:ResetOverride()
    self.override = {};
end

function EffectSettings:AddOverrideEffect(effect)
    self.override[effect] = true;
end

function EffectSettings:RemoveOverrideEffect(effect)
    self.override[effect] = nil;
end

function EffectSettings:IsOverrideEffect(effect)
    return self.override[effect];
end

function EffectSettings:IsEffectFiltered(effect)
    -- is the effect on the override list
    local name = effect:GetName();
    if (self.override[name]) then
        return false;
    end
    
    -- is this type filtered?
    local debuff = effect:IsDebuff();
    local curable = effect:IsCurable();
    if (self.showCurableOnly and (not curable) and debuff) then
        return true;
    end
    
    -- is this category filtered
    local category = effect:GetCategory();
    if((category == Turbine.Gameplay.EffectCategory.Disease) and (not self.showDisease)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Fear) and (not self.showFear)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Wound) and (not self.showWound)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Poison) and (not self.showPoison)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Physical) and (not self.showPhysical)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Elemental) and (not self.showElemental)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Song) and (not self.showSong)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Corruption) and (not self.showCorruption)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Cry) and (not self.showCry)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Undefined) and (not self.showUndefined)) then
        return true;
    end
    
    -- is the duration filtered
    local duration = effect:GetDuration();
    if (self.minDuration ~= 0 and duration <= self.minDuration) then
        return true;
    end
    if (self.maxDuration ~= 0 and duration >= self.maxDuration) then
        return true;
    end
    
    -- is the name filtered
    if (self.ignored[name]) then
        return true;
    end
    
    -- not filtered
    return false;
end

function EffectSettings:IsPotFiltered(pot)
    -- is this category filtered
    local category = pot:GetCategory();
    if((category == PengorosPlugins.Utils.CoolDownCategory.Disease) and (not self.showDiseaseCD)) then
        return true;
    elseif((category == PengorosPlugins.Utils.CoolDownCategory.Fear) and (not self.showFearCD)) then
        return true;
    elseif((category == PengorosPlugins.Utils.CoolDownCategory.Poison) and (not self.showPoisonCD)) then
        return true;
    elseif((category == PengorosPlugins.Utils.CoolDownCategory.Wound) and (not self.showWoundCD)) then
        return true;
    elseif((category == PengorosPlugins.Utils.CoolDownCategory.Morale) and (not self.showMoraleCD)) then
        return true;
    elseif((category == PengorosPlugins.Utils.CoolDownCategory.Power) and (not self.showPowerCD)) then
        return true;
    end

    -- not filtered
    return false;
end
