import "Turbine.UI";

OptionsLogPanel = class(Turbine.UI.ListBox);

-- helper function
local tt = function(width, height, control, text)
    local tooltip = PengorosPlugins.UI.Tooltip();
    tooltip:SetText(text);
    tooltip:SetControl(control);
    tooltip:SetSize(width, height);
end

local holder = function(parent, height)
    local container = Turbine.UI.Control();
    container:SetSize(400, height);
    parent:AddItem(container);
    return container;
end

function OptionsLogPanel:Constructor()
    Turbine.UI.ListBox.Constructor(self);

    self:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self:SetMaxItemsPerLine(1);

    local line = holder(self, 35);
    self.generalTitle = PengorosPlugins.UI.PanelDivider();
    self.generalTitle:SetParent(line);
    self.generalTitle:SetText(L["General"]);
    self.generalTitle:SetSize(400, 30);

    line = holder(self, 20);
    self.enabledCB = PengorosPlugins.UI.CheckBox();
    self.enabledCB:SetParent(line);
    self.enabledCB:SetText(L["Enabled"]);
    self.enabledCB:SetSize(400, 20);
    tt(475, 19, self.enabledCB, L["When checked, effects will be logged to the chat window."]);
    self.enabledCB.CheckedChanged = function(sender, args)
        self:EnableControls(sender:IsChecked());
    end

    line = holder(self, 20);
    self.showBuffsCB = PengorosPlugins.UI.CheckBox();
    self.showBuffsCB:SetParent(line);
    self.showBuffsCB:SetText(L["Buffs"]);
    self.showBuffsCB:SetSize(400, 20);
    tt(300, 19, self.showBuffsCB, L["When checked, buffs will be logged."]);

    line = holder(self, 25);
    self.showDebuffsCB = PengorosPlugins.UI.CheckBox();
    self.showDebuffsCB:SetParent(line);
    self.showDebuffsCB:SetText(L["Debuffs"]);
    self.showDebuffsCB:SetSize(400, 20);
    tt(315, 19, self.showDebuffsCB, L["When checked, debuffs will be logged."]);

    line = holder(self, 35);
    self.filterTitle = PengorosPlugins.UI.PanelDivider();
    self.filterTitle:SetParent(line);
    self.filterTitle:SetText(L["Log Format"]);
    self.filterTitle:SetSize(400, 30);

    line = holder(self, 20);
    self.logIdCB = PengorosPlugins.UI.CheckBox();
    self.logIdCB:SetParent(line);
    self.logIdCB:SetText(L["Id"]);
    self.logIdCB:SetSize(400, 20);
    tt(355, 19, self.logIdCB, L["When checked, the effect id will be logged."]);

    line = holder(self, 20);
    self.logNameCB = PengorosPlugins.UI.CheckBox();
    self.logNameCB:SetParent(line);
    self.logNameCB:SetText(L["Name"]);
    self.logNameCB:SetSize(400, 20);
    tt(375, 19, self.logNameCB, L["When checked, the effect name will be logged."]);

    line = holder(self, 20);
    self.logDescriptionCB = PengorosPlugins.UI.CheckBox();
    self.logDescriptionCB:SetParent(line);
    self.logDescriptionCB:SetText(L["Description"]);
    self.logDescriptionCB:SetSize(400, 20);
    tt(430, 19, self.logDescriptionCB, L["When checked, the effect description will be logged."]);

    line = holder(self, 20);
    self.logCategoryCB = PengorosPlugins.UI.CheckBox();
    self.logCategoryCB:SetParent(line);
    self.logCategoryCB:SetText(L["Category"]);
    self.logCategoryCB:SetSize(400, 20);
    tt(410, 19, self.logCategoryCB, L["When checked, the effect category will be logged."]);

    line = holder(self, 20);
    self.logDurationCB = PengorosPlugins.UI.CheckBox();
    self.logDurationCB:SetParent(line);
    self.logDurationCB:SetText(L["Duration"]);
    self.logDurationCB:SetSize(400, 20);
    tt(410, 19, self.logDurationCB, L["When checked, the effect duration will be logged."]);

    line = holder(self, 20);
    self.logCurableCB = PengorosPlugins.UI.CheckBox();
    self.logCurableCB:SetParent(line);
    self.logCurableCB:SetText(L["Is Curable"]);
    self.logCurableCB:SetSize(400, 20);
    tt(450, 19, self.logCurableCB, L["When checked, the effect curable status will be logged."]);

    line = holder(self, 20);
    self.logDebuffCB = PengorosPlugins.UI.CheckBox();
    self.logDebuffCB:SetParent(line);
    self.logDebuffCB:SetText(L["Is Debuff"]);
    self.logDebuffCB:SetSize(400, 20);
    tt(440, 19, self.logDebuffCB, L["When checked, the effect debuff status will be logged."]);
end

function OptionsLogPanel:EnableControls(enable)
    self.logIdCB:SetEnabled(enable);
    self.logNameCB:SetEnabled(enable);
    self.logDescriptionCB:SetEnabled(enable);
    self.logCategoryCB:SetEnabled(enable);
    self.logDurationCB:SetEnabled(enable);
    self.logCurableCB:SetEnabled(enable);
    self.logDebuffCB:SetEnabled(enable);
    self.showBuffsCB:SetEnabled(enable);
    self.showDebuffsCB:SetEnabled(enable);
end

function OptionsLogPanel:SetOptions()
    local settings = Settings.instance.log;
    self.enabledCB:SetChecked(settings.enabled);
    self.logIdCB:SetChecked(settings.logId);
    self.logNameCB:SetChecked(settings.logName);
    self.logDescriptionCB:SetChecked(settings.logDescription);
    self.logCategoryCB:SetChecked(settings.logCategory);
    self.logDurationCB:SetChecked(settings.logDuration);
    self.logCurableCB:SetChecked(settings.logCurable);
    self.logDebuffCB:SetChecked(settings.logDebuff);
    self.showBuffsCB:SetChecked(settings.showBuffs);
    self.showDebuffsCB:SetChecked(settings.showDebuffs);
end

function OptionsLogPanel:SaveOptions()
    local settings = Settings.instance.log;
    settings.enabled = self.enabledCB:IsChecked();
    settings.logId = self.logIdCB:IsChecked();
    settings.logName = self.logNameCB:IsChecked();
    settings.logDescription = self.logDescriptionCB:IsChecked();
    settings.logCategory = self.logCategoryCB:IsChecked();
    settings.logDuration = self.logDurationCB:IsChecked();
    settings.logCurable = self.logCurableCB:IsChecked();
    settings.logDebuff = self.logDebuffCB:IsChecked();
    settings.showBuffs = self.showBuffsCB:IsChecked();
    settings.showDebuffs = self.showDebuffsCB:IsChecked();
end
