import "Turbine.UI";

OptionsQuickslotPanel = class(Turbine.UI.ListBox);

-- helper functions
local tt = function(width, height, control, text)
    local tooltip = PengorosPlugins.UI.Tooltip();
    tooltip:SetText(text);
    tooltip:SetControl(control);
    tooltip:SetSize(width, height);
end

local holder = function(parent, height)
    local container = Turbine.UI.Control();
    container:SetSize(400, height);
    parent:AddItem(container);
    return container;
end

local setShortcut = function(quickslot, shortcutType, shortcutData)
    local setter = function()
        if (shortcutData ~= "") then
            local shortcut = Turbine.UI.Lotro.Shortcut(shortcutType, shortcutData);
            quickslot:SetShortcut(shortcut);
        else
            quickslot:SetShortcut(nil);
        end
    end
    
    if (not pcall(setter)) then
        quickslot:SetShortcut(nil);
    end
end

function OptionsQuickslotPanel:Constructor()
    Turbine.UI.ListBox.Constructor(self);

    self.current = nil;
    self:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self:SetMaxItemsPerLine(1);

    local line = holder(self, 35);
    self.barTitle = PengorosPlugins.UI.PanelDivider();
    self.barTitle:SetParent(line);
    self.barTitle:SetText(L["Quickslot Bars"]);
    self.barTitle:SetSize(400, 30);
    
    line = holder(self, 30);
    self.barLabel = PengorosPlugins.UI.Label();
    self.barLabel:SetParent(line);
    self.barLabel:SetSize(176, 20);
    self.barLabel:SetPosition(20, 0);
    self.barLabel:SetText(L["Bar"]);
    tt(415, 19, self.barLabel, L["Select the Quickslot Bar you would like to modify."]);
    
    self.barCombo = PengorosPlugins.UI.ComboBox();
    self.barCombo:SetParent(line);
    self.barCombo:SetSize(204, 20);
    self.barCombo:SetPosition(196, 0);
    self.barCombo.SelectedIndexChanged = function(sender, args)
        if (self.current ~= nil) then
            self:SaveBarOptions(self.current);
        end
        self:SetBarOptions(args.selection);
        self.current = args.selection;
    end

    line = holder(self, 30);
    self.newButton  = Turbine.UI.Lotro.Button();
    self.newButton:SetParent(line);
    self.newButton:SetPosition(89, 0);
    self.newButton:SetSize(100, 22);
    self.newButton:SetText(L["New"]);
    tt(230, 19, self.newButton, L["Create a new Quickslot Bar."]);
    self.newButton.Click = function(sender, args)
        self:AddNewBar();
    end

    self.copyButton  = Turbine.UI.Lotro.Button();
    self.copyButton:SetParent(line);
    self.copyButton:SetPosition(196, 0);
    self.copyButton:SetSize(100, 22);
    self.copyButton:SetText(L["Copy"]);
    tt(270, 19, self.copyButton, L["Copy the current Quickslot Bar."]);
    self.copyButton.Click = function(sender, args)
        self:CopyCurrentBar();
    end

    self.removeButton  = Turbine.UI.Lotro.Button();
    self.removeButton:SetParent(line);
    self.removeButton:SetPosition(303, 0);
    self.removeButton:SetSize(100, 22);
    self.removeButton:SetText(L["Remove"]);
    tt(285, 19, self.removeButton, L["Remove the current Quickslot Bar."]);
    self.removeButton.Click = function(sender, args)
        self:RemoveCurrentBar();
    end

    line = holder(self, 35);
    self.generalTitle = PengorosPlugins.UI.PanelDivider();
    self.generalTitle:SetParent(line);
    self.generalTitle:SetText(L["General"]);
    self.generalTitle:SetSize(400, 30);

    line = holder(self, 30);
    self.nameLabel = PengorosPlugins.UI.Label();
    self.nameLabel:SetParent(line);
    self.nameLabel:SetSize(100, 20);
    self.nameLabel:SetPosition(20, 0);
    self.nameLabel:SetText(L["Name"]);
    tt(415, 19, self.nameLabel, L["The descriptive name of the current Quickslot Bar."]);
    
    self.nameText = Turbine.UI.Lotro.TextBox();
    self.nameText:SetParent(line);
    self.nameText:SetSize(280, 20);
    self.nameText:SetPosition(120, 0);
    self.nameText:SetFont(Turbine.UI.Lotro.Font.Verdana14);
    self.nameText:SetMultiline(false);
    self.nameText:SetWantsUpdates(true);
    self.nameText.Update = function(sender, args)
        self.barCombo:UpdateSelectionText(self.nameText:GetText());
    end

    line = holder(self, 25);
    self.showQuickslotCB = PengorosPlugins.UI.CheckBox();
    self.showQuickslotCB:SetParent(line);
    self.showQuickslotCB:SetText(L["Enabled"]);
    self.showQuickslotCB:SetSize(400, 20);
    tt(395, 19, self.showQuickslotCB, L["When checked, this Quickslot Bar will be enbled."]);

    line = holder(self, 30);
    self.visibleTypeLabel = PengorosPlugins.UI.Label();
    self.visibleTypeLabel:SetParent(line);
    self.visibleTypeLabel:SetSize(190, 20);
    self.visibleTypeLabel:SetPosition(20, 0);
    self.visibleTypeLabel:SetText(L["Show Bar"]);
    tt(240, 19, self.visibleTypeLabel, L["Select when the bar is visible."]);
    
    self.visibleTypeCombo = PengorosPlugins.UI.ComboBox();
    self.visibleTypeCombo:SetParent(line);
    self.visibleTypeCombo:SetSize(190, 20);
    self.visibleTypeCombo:SetPosition(210, 0);
    self.visibleTypeCombo:AddItem(L["Always"], WindowVisibility.Always);
    self.visibleTypeCombo:AddItem(L["Only In Combat"], WindowVisibility.InCombat);
    self.visibleTypeCombo:AddItem(L["Only Out Of Combat"], WindowVisibility.OutOfCombat);

    line = holder(self, 35);
    self.filterTitle = PengorosPlugins.UI.PanelDivider();
    self.filterTitle:SetParent(line);
    self.filterTitle:SetText(L["Filters"]);
    self.filterTitle:SetSize(400, 30);

    line = holder(self, 20);
    self.showDiseaseCB = PengorosPlugins.UI.CheckBox();
    self.showDiseaseCB:SetParent(line);
    self.showDiseaseCB:SetText(L["Disease"]);
    self.showDiseaseCB:SetSize(400, 20);
    tt(420, 19, self.showDiseaseCB, L["When checked, the disease quickslot will be shown."]);

    line = holder(self, 20);
    self.showFearCB = PengorosPlugins.UI.CheckBox();
    self.showFearCB:SetParent(line);
    self.showFearCB:SetText(L["Fear"]);
    self.showFearCB:SetSize(400, 20);
    tt(395, 19, self.showFearCB, L["When checked, the fear quickslot will be shown."]);

    line = holder(self, 20);
    self.showPoisonCB = PengorosPlugins.UI.CheckBox();
    self.showPoisonCB:SetParent(line);
    self.showPoisonCB:SetText(L["Poison"]);
    self.showPoisonCB:SetSize(400, 20);
    tt(420, 19, self.showPoisonCB, L["When checked, the poison quickslot will be shown."]);

    line = holder(self, 20);
    self.showWoundCB = PengorosPlugins.UI.CheckBox();
    self.showWoundCB:SetParent(line);
    self.showWoundCB:SetText(L["Wound"]);
    self.showWoundCB:SetSize(400, 20);
    tt(420, 19, self.showWoundCB, L["When checked, the wound quickslot will be shown."]);

    line = holder(self, 20);
    self.showMoraleCB = PengorosPlugins.UI.CheckBox();
    self.showMoraleCB:SetParent(line);
    self.showMoraleCB:SetText(L["Morale"]);
    self.showMoraleCB:SetSize(400, 20);
    tt(420, 19, self.showMoraleCB, L["When checked, the morale quickslot will be shown."]);

    line = holder(self, 30);
    self.showPowerCB = PengorosPlugins.UI.CheckBox();
    self.showPowerCB:SetParent(line);
    self.showPowerCB:SetText(L["Power"]);
    self.showPowerCB:SetSize(400, 20);
    tt(415, 19, self.showPowerCB, L["When checked, the power quickslot will be shown."]);

    line = holder(self, 50);
    self.moraleSlider = PengorosPlugins.UI.Slider();
    self.moraleSlider:SetParent(line);
    self.moraleSlider:SetText(L["Morale Percent"]);
    self.moraleSlider:SetPosition(50, 0);
    self.moraleSlider:SetSize(350, 40);
    self.moraleSlider:SetStep(1);
    self.moraleSlider:SetMin(1);
    self.moraleSlider:SetMax(99);
    tt(480, 38, self.moraleSlider.label, L["Adjust the morale percent threshold.  When your morale drops below this level the quickslot will appear."]);

    line = holder(self, 50);
    self.powerSlider = PengorosPlugins.UI.Slider();
    self.powerSlider:SetParent(line);
    self.powerSlider:SetText(L["Power Percent"]);
    self.powerSlider:SetPosition(50, 0);
    self.powerSlider:SetSize(350, 40);
    self.powerSlider:SetStep(1);
    self.powerSlider:SetMin(1);
    self.powerSlider:SetMax(99);
    tt(480, 38, self.powerSlider.label, L["Adjust the power percent threshold.  When your power drops below this level the quickslot will appear."]);

    line = holder(self, 35);
    self.quickslotTitle = PengorosPlugins.UI.PanelDivider();
    self.quickslotTitle:SetParent(line);
    self.quickslotTitle:SetText(L["Quick Slots"]);
    self.quickslotTitle:SetSize(400, 30);

    line = holder(self, 40);
    self.moraleQuickslot = Turbine.UI.Lotro.Quickslot();
    self.moraleQuickslot:SetParent(line);
    self.moraleQuickslot:SetSize(36, 36);
    self.moraleQuickslot:SetPosition(99, 0);
    
    self.powerQuickslot = Turbine.UI.Lotro.Quickslot();
    self.powerQuickslot:SetParent(line);
    self.powerQuickslot:SetSize(36, 36);
    self.powerQuickslot:SetPosition(192, 0);

    self.diseaseQuickslot = Turbine.UI.Lotro.Quickslot();
    self.diseaseQuickslot:SetParent(line);
    self.diseaseQuickslot:SetSize(36, 36);
    self.diseaseQuickslot:SetPosition(285, 0);
    
    line = holder(self, 20);
    self.moraleLabel = PengorosPlugins.UI.Label();
    self.moraleLabel:SetParent(line);
    self.moraleLabel:SetSize(92, 20);
    self.moraleLabel:SetPosition(71, 0);
    self.moraleLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
    self.moraleLabel:SetText(L["Morale"]);
    tt(480, 38, self.moraleLabel, L["Drag an item here to make a shortcut that will be shown when your morale is low."]);
    
    self.powerLabel = PengorosPlugins.UI.Label();
    self.powerLabel:SetParent(line);
    self.powerLabel:SetSize(92, 20);
    self.powerLabel:SetPosition(164, 0);
    self.powerLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
    self.powerLabel:SetText(L["Power"]);
    tt(480, 38, self.powerLabel, L["Drag an item here to make a shortcut that will be shown when your power is low."]);

    self.diseaseLabel = PengorosPlugins.UI.Label();
    self.diseaseLabel:SetParent(line);
    self.diseaseLabel:SetSize(92, 20);
    self.diseaseLabel:SetPosition(257, 0);
    self.diseaseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
    self.diseaseLabel:SetText(L["Disease"]);
    tt(480, 38, self.diseaseLabel, L["Drag an item here to make a shortcut that will be shown when you have a curable disease."]);

    line = holder(self, 40);
    self.fearQuickslot = Turbine.UI.Lotro.Quickslot();
    self.fearQuickslot:SetParent(line);
    self.fearQuickslot:SetSize(36, 36);
    self.fearQuickslot:SetPosition(99, 0);
    
    self.poisonQuickslot = Turbine.UI.Lotro.Quickslot();
    self.poisonQuickslot:SetParent(line);
    self.poisonQuickslot:SetSize(36, 36);
    self.poisonQuickslot:SetPosition(192, 0);

    self.woundQuickslot = Turbine.UI.Lotro.Quickslot();
    self.woundQuickslot:SetParent(line);
    self.woundQuickslot:SetSize(36, 36);
    self.woundQuickslot:SetPosition(285, 0);
    
    line = holder(self, 20);
    self.fearLabel = PengorosPlugins.UI.Label();
    self.fearLabel:SetParent(line);
    self.fearLabel:SetSize(92, 20);
    self.fearLabel:SetPosition(71, 0);
    self.fearLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
    self.fearLabel:SetText(L["Fear"]);
    tt(480, 38, self.fearLabel, L["Drag an item here to make a shortcut that will be shown when you have a curable fear."]);
    
    self.poisonLabel = PengorosPlugins.UI.Label();
    self.poisonLabel:SetParent(line);
    self.poisonLabel:SetSize(92, 20);
    self.poisonLabel:SetPosition(164, 0);
    self.poisonLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
    self.poisonLabel:SetText(L["Poison"]);
    tt(480, 38, self.poisonLabel, L["Drag an item here to make a shortcut that will be shown when you have a curable poison."]);

    self.woundLabel = PengorosPlugins.UI.Label();
    self.woundLabel:SetParent(line);
    self.woundLabel:SetSize(92, 20);
    self.woundLabel:SetPosition(257, 0);
    self.woundLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
    self.woundLabel:SetText(L["Wound"]);
    tt(480, 38, self.woundLabel, L["Drag an item here to make a shortcut that will be shown when you have a curable wound."]);
end

function OptionsQuickslotPanel:AddNewBar()
    local bar = QuickslotSettings();
    bar.name = L["New Quickslot"];
    self.barCombo:AddItem(bar.name, bar);
    self.barCombo:SetSelection(bar);
    self:EnableControls(true);
end

function OptionsQuickslotPanel:CopyCurrentBar()
    self:SaveBarOptions(self.current); -- make sure the options are up to date 
    local bar = self.current:Copy();
    bar.name = bar.name .. " - " .. L["Copy"];
    self.barCombo:AddItem(bar.name, bar);
    self.barCombo:SetSelection(bar);
    self:EnableControls(true);
end

function OptionsQuickslotPanel:RemoveCurrentBar()
    self.barCombo:RemoveItem(self.current);
    self:EnableControls(self.barCombo:GetItemCount() > 0);
end

function OptionsQuickslotPanel:EnableControls(enable)
    self.barCombo:SetEnabled(enable);
    self.copyButton:SetEnabled(enable);
    self.removeButton:SetEnabled(enable);
    self.nameLabel:SetEnabled(enable);
    self.nameText:SetEnabled(enable);
    self.showQuickslotCB:SetEnabled(enable);
    self.visibleTypeLabel:SetEnabled(enable);
    self.visibleTypeCombo:SetEnabled(enable);
    self.showDiseaseCB:SetEnabled(enable);
    self.showFearCB:SetEnabled(enable);
    self.showPoisonCB:SetEnabled(enable);
    self.showWoundCB:SetEnabled(enable);
    self.showMoraleCB:SetEnabled(enable);
    self.showPowerCB:SetEnabled(enable);
    self.moraleSlider:SetEnabled(enable);
    self.powerSlider:SetEnabled(enable);
    self.moraleLabel:SetEnabled(enable);
    self.powerLabel:SetEnabled(enable);
    self.diseaseLabel:SetEnabled(enable);
    self.fearLabel:SetEnabled(enable);
    self.poisonLabel:SetEnabled(enable);
    self.woundLabel:SetEnabled(enable);
    self.moraleQuickslot:SetVisible(enable);
    self.powerQuickslot:SetVisible(enable);
    self.diseaseQuickslot:SetVisible(enable);
    self.fearQuickslot:SetVisible(enable);
    self.poisonQuickslot:SetVisible(enable);
    self.woundQuickslot:SetVisible(enable);
end

function OptionsQuickslotPanel:SetOptions()
    -- bars
    local i;
    local size = table.getn(Settings.instance.quickslots);
    for i = 1, size do
        local bar = Settings.instance.quickslots[i]:Copy();
        self.barCombo:AddItem(bar.name, bar);
    end
    
    if (size > 0) then
        self.barCombo:SetSelection(self.barCombo:GetItem(1));
    else
        self:EnableControls(false);
    end
end

function OptionsQuickslotPanel:SaveOptions()
    -- save the current bar settings
    if (self.current ~= nil) then
        self:SaveBarOptions(self.current);
    end
    
    -- collect all the bars into an array
    local bars = {};
    local i;
    for i = 1, self.barCombo:GetItemCount() do
        bars[i] = self.barCombo:GetItem(i);
    end
    
    -- overwrite the existing quickslots
    Settings.instance.quickslots = bars;
end

function OptionsQuickslotPanel:SetBarOptions(settings)
    -- quickslots
    self.nameText:SetText(settings.name);
    self.showQuickslotCB:SetChecked(settings.enabled);
    self.visibleTypeCombo:SetSelection(settings.visibleType);
    
    self.showDiseaseCB:SetChecked(settings.showDisease);
    self.showFearCB:SetChecked(settings.showFear);
    self.showWoundCB:SetChecked(settings.showWound);
    self.showPoisonCB:SetChecked(settings.showPoison);
    self.showMoraleCB:SetChecked(settings.showMorale);
    self.showPowerCB:SetChecked(settings.showPower);
    self.moraleSlider:SetValue(settings.moralePercent * 100);
    self.powerSlider:SetValue(settings.powerPercent * 100);

    -- shortcuts, set the first one on a timer because there appears to be a timing issue with listboxes and quickslots on the first view
    local set = function()
        setShortcut(self.moraleQuickslot, settings.moraleType, settings.moraleData);
        setShortcut(self.powerQuickslot, settings.powerType, settings.powerData);
        setShortcut(self.diseaseQuickslot, settings.diseaseType, settings.diseaseData);
        setShortcut(self.fearQuickslot, settings.fearType, settings.fearData);
        setShortcut(self.poisonQuickslot, settings.poisonType, settings.poisonData);
        setShortcut(self.woundQuickslot, settings.woundType, settings.woundData);
    end
    
    if (self.timer == nil) then
        self.timer = PengorosPlugins.Utils.Timer(1);
        self.timer.Signal = function(sender, args)
            set();
        end
        self.timer:Start();
    else
        set();
    end
end

function OptionsQuickslotPanel:SaveBarOptions(settings)
    -- quickslots
    settings.name = self.nameText:GetText();
    settings.enabled = self.showQuickslotCB:IsChecked();
    settings.visibleType = self.visibleTypeCombo:GetSelection();

    settings.showDisease = self.showDiseaseCB:IsChecked();
    settings.showFear = self.showFearCB:IsChecked();
    settings.showWound = self.showWoundCB:IsChecked();
    settings.showPoison = self.showPoisonCB:IsChecked();
    settings.showMorale = self.showMoraleCB:IsChecked();
    settings.showPower = self.showPowerCB:IsChecked();
    settings.moralePercent = self.moraleSlider:GetValue() / 100;
    settings.powerPercent = self.powerSlider:GetValue() / 100;
    
    local shortcut = self.moraleQuickslot:GetShortcut();
    if (shortcut:GetData() ~= "") then
        settings.moraleType = shortcut:GetType();
        settings.moraleData = shortcut:GetData();
    end
    shortcut = self.powerQuickslot:GetShortcut();
    if (shortcut:GetData() ~= "") then
        settings.powerType = shortcut:GetType();
        settings.powerData = shortcut:GetData();
    end
    shortcut = self.diseaseQuickslot:GetShortcut();
    if (shortcut:GetData() ~= "") then
        settings.diseaseType = shortcut:GetType();
        settings.diseaseData = shortcut:GetData();
    end
    shortcut = self.fearQuickslot:GetShortcut();
    if (shortcut:GetData() ~= "") then
        settings.fearType = shortcut:GetType();
        settings.fearData = shortcut:GetData();
    end
    shortcut = self.poisonQuickslot:GetShortcut();
    if (shortcut:GetData() ~= "") then
        settings.poisonType = shortcut:GetType();
        settings.poisonData = shortcut:GetData();
    end
    shortcut = self.woundQuickslot:GetShortcut();
    if (shortcut:GetData() ~= "") then
        settings.woundType = shortcut:GetType();
        settings.woundData = shortcut:GetData();
    end
end

function OptionsQuickslotPanel:Destroy()
    -- cleanup the quickslots so the client doesn't crash
    self.moraleQuickslot:SetShortcut(nil);
    self.powerQuickslot:SetShortcut(nil);
    self.diseaseQuickslot:SetShortcut(nil);
    self.fearQuickslot:SetShortcut(nil);
    self.poisonQuickslot:SetShortcut(nil);
    self.woundQuickslot:SetShortcut(nil);
end
