import "Turbine.UI";

OptionsSliderPanel = class(Turbine.UI.ListBox);

OptionsSliderPanel.Border = 1;
OptionsSliderPanel.Background = 2;
OptionsSliderPanel.Font = 3;
OptionsSliderPanel.Outline = 4;

-- helper functions
local tt = function(width, height, control, text)
    local tooltip = PengorosPlugins.UI.Tooltip();
    tooltip:SetText(text);
    tooltip:SetControl(control);
    tooltip:SetSize(width, height);
end

local holder = function(parent, height)
    local container = Turbine.UI.Control();
    container:SetSize(400, height);
    parent:AddItem(container);
    return container;
end

function OptionsSliderPanel:Constructor()
    Turbine.UI.ListBox.Constructor(self);

    self.current = nil;
    self.ignoreWasVisible = false;
    self.overrideWasVisible = false;
    self:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self:SetMaxItemsPerLine(1);
    
    -- listen for events
    PengorosPlugins.Utils.KeyManager.AddListener(self);

    local line = holder(self, 35);
    self.sliderTitle = PengorosPlugins.UI.PanelDivider();
    self.sliderTitle:SetParent(line);
    self.sliderTitle:SetText(L["Effect Sliders"]);
    self.sliderTitle:SetSize(400, 30);
    
    line = holder(self, 30);
    self.sliderLabel = PengorosPlugins.UI.Label();
    self.sliderLabel:SetParent(line);
    self.sliderLabel:SetSize(176, 20);
    self.sliderLabel:SetPosition(20, 0);
    self.sliderLabel:SetText(L["Slider"]);
    tt(400, 19, self.sliderLabel, L["Select the Effect Slider you would like to modify."]);
    
    self.sliderCombo = PengorosPlugins.UI.ComboBox();
    self.sliderCombo:SetParent(line);
    self.sliderCombo:SetSize(204, 20);
    self.sliderCombo:SetPosition(196, 0);
    self.sliderCombo.SelectedIndexChanged = function(sender, args)
        if (self.current ~= nil) then
            self:SaveSliderOptions(self.current);
        end
        self:SetSliderOptions(args.selection);
        self.current = args.selection;
    end

    line = holder(self, 30);    
    self.newButton  = Turbine.UI.Lotro.Button();
    self.newButton:SetParent(line);
    self.newButton:SetPosition(89, 0);
    self.newButton:SetSize(100, 22);
    self.newButton:SetText(L["New"]);
    tt(220, 19, self.newButton, L["Create a new Effect Slider."]);
    self.newButton.Click = function(sender, args)
        self:AddNewSlider();
    end

    self.copyButton  = Turbine.UI.Lotro.Button();
    self.copyButton:SetParent(line);
    self.copyButton:SetPosition(196, 0);
    self.copyButton:SetSize(100, 22);
    self.copyButton:SetText(L["Copy"]);
    tt(255, 19, self.copyButton, L["Copy the current Effect Slider."]);
    self.copyButton.Click = function(sender, args)
        self:CopyCurrentSlider();
    end

    self.removeButton  = Turbine.UI.Lotro.Button();
    self.removeButton:SetParent(line);
    self.removeButton:SetPosition(303, 0);
    self.removeButton:SetSize(100, 22);
    self.removeButton:SetText(L["Remove"]);
    tt(275, 19, self.removeButton, L["Remove the current Effect Slider."]);
    self.removeButton.Click = function(sender, args)
        self:RemoveCurrentSlider();
    end

    line = holder(self, 35);
    self.generalTitle = PengorosPlugins.UI.PanelDivider();
    self.generalTitle:SetParent(line);
    self.generalTitle:SetText(L["General"]);
    self.generalTitle:SetSize(400, 30);

    line = holder(self, 30);    
    self.nameLabel = PengorosPlugins.UI.Label();
    self.nameLabel:SetParent(line);
    self.nameLabel:SetSize(100, 20);
    self.nameLabel:SetPosition(20, 0);
    self.nameLabel:SetText(L["Name"]);
    tt(405, 19, self.nameLabel, L["The descriptive name of the current Effect Slider."]);
    
    self.nameText = Turbine.UI.Lotro.TextBox();
    self.nameText:SetParent(line);
    self.nameText:SetSize(280, 20);
    self.nameText:SetPosition(120, 0);
    self.nameText:SetFont(Turbine.UI.Lotro.Font.Verdana14);
    self.nameText:SetMultiline(false);
    self.nameText:SetWantsUpdates(true);
    self.nameText.Update = function(sender, args)
        self.sliderCombo:UpdateSelectionText(self.nameText:GetText());
    end
    
    line = holder(self, 20);    
    self.showSliderCB = PengorosPlugins.UI.CheckBox();
    self.showSliderCB:SetParent(line);
    self.showSliderCB:SetText(L["Enabled"]);
    self.showSliderCB:SetSize(400, 20);
    tt(390, 19, self.showSliderCB, L["When checked, this Effect Slider will be enbled."]);

    line = holder(self, 20);    
    self.showDebuffsCB = PengorosPlugins.UI.CheckBox();
    self.showDebuffsCB:SetParent(line);
    self.showDebuffsCB:SetText(L["Debuff Effects"]);
    self.showDebuffsCB:SetSize(400, 20);
    tt(445, 19, self.showDebuffsCB, L["When checked, debuff effects will appear on the slider."]);
    
    line = holder(self, 25);    
    self.showBuffsCB = PengorosPlugins.UI.CheckBox();
    self.showBuffsCB:SetParent(line);
    self.showBuffsCB:SetText(L["Buff Effects"]);
    self.showBuffsCB:SetSize(400, 20);
    tt(425, 19, self.showBuffsCB, L["When checked, buff effects will appear on the slider."]);

--[[
    line = holder(self, 25);    
    self.showCoolDownsCB = PengorosPlugins.UI.CheckBox();
    self.showCoolDownsCB:SetParent(line);
    self.showCoolDownsCB:SetText(L["Cool Down Items"]);
    self.showCoolDownsCB:SetSize(400, 20);
    tt(420, 19, self.showCoolDownsCB, L["When checked cooldown timers will appear on the slider."]);
--]]

    line = holder(self, 30);    
    self.visibleTypeLabel = PengorosPlugins.UI.Label();
    self.visibleTypeLabel:SetParent(line);
    self.visibleTypeLabel:SetSize(190, 20);
    self.visibleTypeLabel:SetPosition(20, 0);
    self.visibleTypeLabel:SetText(L["Show Slider"]);
    tt(260, 19, self.visibleTypeLabel, L["Select when the slider is visible."]);
    
    self.visibleTypeCombo = PengorosPlugins.UI.ComboBox();
    self.visibleTypeCombo:SetParent(line);
    self.visibleTypeCombo:SetSize(190, 20);
    self.visibleTypeCombo:SetPosition(210, 0);
    self.visibleTypeCombo:AddItem(L["Always"], WindowVisibility.Always);
    self.visibleTypeCombo:AddItem(L["Only In Combat"], WindowVisibility.InCombat);
    self.visibleTypeCombo:AddItem(L["Only Out Of Combat"], WindowVisibility.OutOfCombat);
    self.visibleTypeCombo:AddItem(L["Has Effects"], WindowVisibility.HasEffects);

    line = holder(self, 50);    
    self.timeMaxSlider = PengorosPlugins.UI.Slider();
    self.timeMaxSlider:SetParent(line);
    self.timeMaxSlider:SetText(L["Maximum Time (seconds)"]);
    self.timeMaxSlider:SetPosition(49, 0);
    self.timeMaxSlider:SetSize(350, 40);
    self.timeMaxSlider:SetStep(1);
    self.timeMaxSlider:SetMin(10);
    self.timeMaxSlider:SetMax(1800);
    tt(480, 57, self.timeMaxSlider.label, L["Adjust the maximum time to display in seconds.  Effects longer than this will sit at the end of the bar until they drop below the threshold."]);

    line = holder(self, 50);    
    self.timeCompressionSlider = PengorosPlugins.UI.Slider();
    self.timeCompressionSlider:SetParent(line);
    self.timeCompressionSlider:SetText(L["Time Compression"]);
    self.timeCompressionSlider:SetPosition(49, 0);
    self.timeCompressionSlider:SetSize(350, 40);
    self.timeCompressionSlider:SetStep(0.005);
    self.timeCompressionSlider:SetMin(0.01);
    self.timeCompressionSlider:SetMax(1.0);
    self.timeCompressionSlider:SetFormat("%.3f");
    tt(480, 57, self.timeCompressionSlider.label, L["Adjust the time display scaling factor.  Setting this to 1 will result in linear scaling, as you get closer to 0 larger time values are compressed into a smaller area."]);

    line = holder(self, 35);    
    self.filterTitle = PengorosPlugins.UI.PanelDivider();
    self.filterTitle:SetParent(line);
    self.filterTitle:SetText(L["Effect Filters"]);
    self.filterTitle:SetSize(400, 30);
    
    line = holder(self, 20);    
    self.showCurableOnlyCB = PengorosPlugins.UI.CheckBox();
    self.showCurableOnlyCB:SetParent(line);
    self.showCurableOnlyCB:SetText(L["Curable Only"]);
    self.showCurableOnlyCB:SetSize(400, 20);
    tt(410, 19, self.showCurableOnlyCB, L["When checked, only curable effects will be shown."]);

    line = holder(self, 20);    
    self.showDiseaseCB = PengorosPlugins.UI.CheckBox();
    self.showDiseaseCB:SetParent(line);
    self.showDiseaseCB:SetText(L["Disease"]);
    self.showDiseaseCB:SetSize(400, 20);
    tt(360, 19, self.showDiseaseCB, L["When checked, disease effects will be shown."]);

    line = holder(self, 20);    
    self.showFearCB = PengorosPlugins.UI.CheckBox();
    self.showFearCB:SetParent(line);
    self.showFearCB:SetText(L["Fear"]);
    self.showFearCB:SetSize(400, 20);
    tt(340, 19, self.showFearCB, L["When checked, fear effects will be shown."]);

    line = holder(self, 20);    
    self.showPoisonCB = PengorosPlugins.UI.CheckBox();
    self.showPoisonCB:SetParent(line);
    self.showPoisonCB:SetText(L["Poison"]);
    self.showPoisonCB:SetSize(400, 20);
    tt(365, 19, self.showPoisonCB, L["When checked, poison effects will be shown."]);

    line = holder(self, 20);    
    self.showWoundCB = PengorosPlugins.UI.CheckBox();
    self.showWoundCB:SetParent(line);
    self.showWoundCB:SetText(L["Wound"]);
    self.showWoundCB:SetSize(400, 20);
    tt(365, 19, self.showWoundCB, L["When checked, wound effects will be shown."]);

    line = holder(self, 20);    
    self.showCorruptionCB = PengorosPlugins.UI.CheckBox();
    self.showCorruptionCB:SetParent(line);
    self.showCorruptionCB:SetText(L["Corruption"]);
    self.showCorruptionCB:SetSize(400, 20);
    tt(400, 19, self.showCorruptionCB, L["When checked, corruption effects will be shown."]);

    line = holder(self, 20);    
    self.showCryCB = PengorosPlugins.UI.CheckBox();
    self.showCryCB:SetParent(line);
    self.showCryCB:SetText(L["Cry"]);
    self.showCryCB:SetSize(400, 20);
    tt(335, 19, self.showCryCB, L["When checked, cry effects will be shown."]);

    line = holder(self, 20);    
    self.showElementalCB = PengorosPlugins.UI.CheckBox();
    self.showElementalCB:SetParent(line);
    self.showElementalCB:SetText(L["Elemental"]);
    self.showElementalCB:SetSize(400, 20);
    tt(385, 19, self.showElementalCB, L["When checked, elemental effects will be shown."]);

    line = holder(self, 20);    
    self.showPhysicalCB = PengorosPlugins.UI.CheckBox();
    self.showPhysicalCB:SetParent(line);
    self.showPhysicalCB:SetText(L["Physical"]);
    self.showPhysicalCB:SetSize(400, 20);
    tt(370, 19, self.showPhysicalCB, L["When checked, physical effects will be shown."]);

    line = holder(self, 20);    
    self.showSongCB = PengorosPlugins.UI.CheckBox();
    self.showSongCB:SetParent(line);
    self.showSongCB:SetText(L["Song"]);
    self.showSongCB:SetSize(400, 20);
    tt(350, 19, self.showSongCB, L["When checked, song effects will be shown."]);

    line = holder(self, 30);    
    self.showUndefinedCB = PengorosPlugins.UI.CheckBox();
    self.showUndefinedCB:SetParent(line);
    self.showUndefinedCB:SetText(L["Undefined"]);
    self.showUndefinedCB:SetSize(400, 20);
    tt(390, 19, self.showUndefinedCB, L["When checked, undefined effects will be shown."]);

    line = holder(self, 50);    
    self.minDurationSlider = PengorosPlugins.UI.Slider();
    self.minDurationSlider:SetParent(line);
    self.minDurationSlider:SetText(L["Minimum Duration (seconds)"]);
    self.minDurationSlider:SetPosition(49, 0);
    self.minDurationSlider:SetSize(350, 40);
    self.minDurationSlider:SetStep(1);
    self.minDurationSlider:SetMin(0);
    self.minDurationSlider:SetMax(3600);
    tt(480, 38, self.minDurationSlider.label, L["Effects shorter than this will not be shown.  Set to 0 to disable."]);

    line = holder(self, 50);    
    self.maxDurationSlider = PengorosPlugins.UI.Slider();
    self.maxDurationSlider:SetParent(line);
    self.maxDurationSlider:SetText(L["Maximum Duration (seconds)"]);
    self.maxDurationSlider:SetPosition(49, 0);
    self.maxDurationSlider:SetSize(350, 40);
    self.maxDurationSlider:SetStep(1);
    self.maxDurationSlider:SetMin(0);
    self.maxDurationSlider:SetMax(3600);
    tt(480, 38, self.maxDurationSlider.label, L["Effects longer than this will not be shown.  Set to 0 to disable."]);

    line = holder(self, 30);    
    self.ignoreWindow = IgnoreWindow();
    self.ignoreButton  = Turbine.UI.Lotro.Button();
    self.ignoreButton:SetParent(line);
    self.ignoreButton:SetPosition(76, 0);
    self.ignoreButton:SetSize(160, 22);
    self.ignoreButton:SetText(L["Ignored Effects"]);
    tt(465, 19, self.ignoreButton, L["Click the button to see a list of effects you have ignored."]);
    self.ignoreButton.Click = function(sender, args)
        self.ignoreWindow:SetVisible(true);
    end

    self.overrideWindow = OverrideWindow();
    self.overrideButton  = Turbine.UI.Lotro.Button();
    self.overrideButton:SetParent(line);
    self.overrideButton:SetPosition(243, 0);
    self.overrideButton:SetSize(160, 22);
    self.overrideButton:SetText(L["Always Shown Effects"]);
    tt(480, 38, self.overrideButton, L["Click the button to see a list of effects that will always be shown."]);
    self.overrideButton.Click = function(sender, args)
        self.overrideWindow:SetVisible(true);
    end

--[[
    line = holder(self, 35);    
    self.cdFilterTitle = PengorosPlugins.UI.PanelDivider();
    self.cdFilterTitle:SetParent(line);
    self.cdFilterTitle:SetText(L["Cool Down Filters"]);
    self.cdFilterTitle:SetSize(400, 30);
    
    line = holder(self, 20);    
    self.showDiseaseCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showDiseaseCoolDownCB:SetParent(line);
    self.showDiseaseCoolDownCB:SetText(L["Disease"]);
    self.showDiseaseCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showDiseaseCoolDownCB, L["When checked, disease pot cooldowns will be shown."]);

    line = holder(self, 20);    
    self.showFearCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showFearCoolDownCB:SetParent(line);
    self.showFearCoolDownCB:SetText(L["Fear"]);
    self.showFearCoolDownCB:SetSize(400, 20);
    tt(415, 19, self.showFearCoolDownCB, L["When checked, fear pot cooldowns will be shown."]);

    line = holder(self, 20);    
    self.showPoisonCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showPoisonCoolDownCB:SetParent(line);
    self.showPoisonCoolDownCB:SetText(L["Poison"]);
    self.showPoisonCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showPoisonCoolDownCB, L["When checked, poison pot cooldowns will be shown."]);

    line = holder(self, 20);    
    self.showWoundCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showWoundCoolDownCB:SetParent(line);
    self.showWoundCoolDownCB:SetText(L["Wound"]);
    self.showWoundCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showWoundCoolDownCB, L["When checked, wound pot cooldowns will be shown."]);

    line = holder(self, 20);    
    self.showMoraleCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showMoraleCoolDownCB:SetParent(line);
    self.showMoraleCoolDownCB:SetText(L["Morale"]);
    self.showMoraleCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showMoraleCoolDownCB, L["When checked, morale pot cooldowns will be shown."]);

    line = holder(self, 30);    
    self.showPowerCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showPowerCoolDownCB:SetParent(line);
    self.showPowerCoolDownCB:SetText(L["Power"]);
    self.showPowerCoolDownCB:SetSize(400, 20);
    tt(430, 19, self.showPowerCoolDownCB, L["When checked, power pot cooldowns will be shown."]);
--]]

    line = holder(self, 35);    
    self.uiTitle = PengorosPlugins.UI.PanelDivider();
    self.uiTitle:SetParent(line);
    self.uiTitle:SetText(L["UI Elements"]);
    self.uiTitle:SetSize(400, 30);

    line = holder(self, 25);    
    self.showEffectTimerCB = PengorosPlugins.UI.CheckBox();
    self.showEffectTimerCB:SetParent(line);
    self.showEffectTimerCB:SetText(L["Effect Timer"]);
    self.showEffectTimerCB:SetSize(400, 20);
    tt(480, 38, self.showEffectTimerCB, L["When checked, the timer for the effect will be placed on the slider."]);
    self.showEffectTimerCB.SetEnabled = function(sender, enabled)
        PengorosPlugins.UI.CheckBox.SetEnabled(sender, enabled);
        self.formatLabel:SetEnabled(enabled and sender:IsChecked());
        self.formatCombo:SetEnabled(enabled and sender:IsChecked());
    end
    self.showEffectTimerCB.CheckedChanged = function(sender, args)
        self.formatLabel:SetEnabled(sender:IsChecked());
        self.formatCombo:SetEnabled(sender:IsChecked());
    end

    line = holder(self, 30);    
    self.formatLabel = PengorosPlugins.UI.Label();
    self.formatLabel:SetParent(line);
    self.formatLabel:SetSize(190, 20);
    self.formatLabel:SetPosition(20, 0);
    self.formatLabel:SetText(L["Timer Format"]);
    tt(330, 19, self.formatLabel, L["Select the display format for the timers."]);
    
    self.formatCombo = PengorosPlugins.UI.ComboBox();
    self.formatCombo:SetParent(line);
    self.formatCombo:SetSize(190, 20);
    self.formatCombo:SetPosition(210, 0);
    self.formatCombo:AddItem(L["Clock: "]..PengorosPlugins.Utils.TimeFormat.CLOCK(75), TimeDisplayFormat.Clock);
    self.formatCombo:AddItem(L["Truncated: "]..PengorosPlugins.Utils.TimeFormat.TRUNCATED(75), TimeDisplayFormat.Truncated);
    self.formatCombo:AddItem(L["Literal: "]..PengorosPlugins.Utils.TimeFormat.LITERAL(75), TimeDisplayFormat.Literal);

    line = holder(self, 30);    
    self.fontLabel = PengorosPlugins.UI.Label();
    self.fontLabel:SetParent(line);
    self.fontLabel:SetSize(190, 20);
    self.fontLabel:SetPosition(20, 0);
    self.fontLabel:SetText(L["Font Size"]);
    tt(325, 19, self.fontLabel, L["Select the font size to use for the text."]);
    
    self.fontCombo = PengorosPlugins.UI.FontComboBox();
    self.fontCombo:SetParent(line);
    self.fontCombo:SetSize(190, 20);
    self.fontCombo:SetPosition(210, 0);
    self.fontCombo:AddItem(L["23px tall"], Turbine.UI.Lotro.Font.Verdana23, 27);
    self.fontCombo:AddItem(L["22px tall"], Turbine.UI.Lotro.Font.Verdana22, 26);
    self.fontCombo:AddItem(L["20px tall"], Turbine.UI.Lotro.Font.Verdana20, 24);
    self.fontCombo:AddItem(L["18px tall"], Turbine.UI.Lotro.Font.Verdana18, 22);
    self.fontCombo:AddItem(L["16px tall"], Turbine.UI.Lotro.Font.Verdana16, 20);
    self.fontCombo:AddItem(L["14px tall"], Turbine.UI.Lotro.Font.Verdana14, 18);
    self.fontCombo:AddItem(L["12px tall"], Turbine.UI.Lotro.Font.Verdana12, 16);
    self.fontCombo:AddItem(L["10px tall"], Turbine.UI.Lotro.Font.Verdana10, 14);

    line = holder(self, 30);    
    self.orientationLabel = PengorosPlugins.UI.Label();
    self.orientationLabel:SetParent(line);
    self.orientationLabel:SetSize(190, 20);
    self.orientationLabel:SetPosition(20, 0);
    self.orientationLabel:SetText(L["Slider Orientation"]);
    tt(360, 19, self.orientationLabel, L["Select the slider orientation and direction."]);
    
    self.orientationCombo = PengorosPlugins.UI.ComboBox();
    self.orientationCombo:SetParent(line);
    self.orientationCombo:SetSize(190, 20);
    self.orientationCombo:SetPosition(210, 0);
    self.orientationCombo:AddItem(L["Left to Right"], SliderOrientation.LeftToRight);
    self.orientationCombo:AddItem(L["Right to Left"], SliderOrientation.RightToLeft);
    self.orientationCombo:AddItem(L["Top to Bottom"], SliderOrientation.TopToBottom);
    self.orientationCombo:AddItem(L["Bottom to Top"], SliderOrientation.BottomToTop);
    self.orientationCombo.SelectedIndexChanged = function(sender, args)
        -- switch the height/width labels to make more sense when talking horizontal/vertical
        if (args.selection <= SliderOrientation.RightToLeft) then
            self.heightSlider:SetText(L["Height"]);
            self.widthSlider:SetText(L["Width"]);
        else
            self.heightSlider:SetText(L["Width"]);
            self.widthSlider:SetText(L["Height"]);
        end
    end    

    line = holder(self, 50);
    self.widthSlider = PengorosPlugins.UI.Slider();
    self.widthSlider:SetParent(line);
    self.widthSlider:SetText(L["Width"]);
    self.widthSlider:SetPosition(49, 0);
    self.widthSlider:SetSize(350, 40);
    self.widthSlider:SetStep(1);
    self.widthSlider:SetMin(50);
    self.widthSlider:SetMax(2560);
    tt(260, 19, self.widthSlider.label, L["Adjust the width of the slider."]);

    line = holder(self, 50);
    self.heightSlider = PengorosPlugins.UI.Slider();
    self.heightSlider:SetParent(line);
    self.heightSlider:SetText(L["Height"]);
    self.heightSlider:SetPosition(49, 0);
    self.heightSlider:SetSize(350, 40);
    self.heightSlider:SetStep(1);
    self.heightSlider:SetMin(10);
    self.heightSlider:SetMax(100);
    tt(260, 19, self.heightSlider.label, L["Adjust the height of the slider."]);

    line = holder(self, 35);
    self.colorTitle = PengorosPlugins.UI.PanelDivider();
    self.colorTitle:SetParent(line);
    self.colorTitle:SetText(L["Color"]);
    self.colorTitle:SetSize(400, 30);

    line = holder(self, 30);
    self.color = Turbine.UI.Control();
    self.color:SetParent(line);
    self.color:SetPosition(60, 0);
    self.color:SetSize(90, 20);
    self.color:SetZOrder(20);
    self.color:SetBackColor(Turbine.UI.Color(1,.8,.8,0));

    self.colorCombo = PengorosPlugins.UI.ComboBox();
    self.colorCombo:SetParent(line);
    self.colorCombo:SetSize(220, 20);
    self.colorCombo:SetPosition(180, 0);
    tt(230, 19, self.colorCombo, L["Select the color to modify."]);
    self.colorCombo:AddItem(L["Border"], OptionsSliderPanel.Border);
    self.colorCombo:AddItem(L["Background"], OptionsSliderPanel.Background);
    self.colorCombo:AddItem(L["Font"], OptionsSliderPanel.Font);
    self.colorCombo:AddItem(L["Font Outline"], OptionsSliderPanel.Outline);
    self.colorCombo.SelectedIndexChanged = function(sender, args)
        self:ChangeColor(self.colors[args.selection]);
    end    

    line = holder(self, 50);
    self.redSlider = PengorosPlugins.UI.Slider();
    self.redSlider:SetParent(line);
    self.redSlider:SetText(L["Red"]);
    self.redSlider:SetPosition(49, 0);
    self.redSlider:SetSize(350, 40);
    self.redSlider:SetStep(1);
    self.redSlider:SetMin(0);
    self.redSlider:SetMax(255);
    tt(350, 19, self.redSlider.label, L["Adjust the red value of the selected color."]);
    self.redSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.r = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end

    line = holder(self, 50);
    self.greenSlider = PengorosPlugins.UI.Slider();
    self.greenSlider:SetParent(line);
    self.greenSlider:SetText(L["Green"]);
    self.greenSlider:SetPosition(49, 0);
    self.greenSlider:SetSize(350, 40);
    self.greenSlider:SetStep(1);
    self.greenSlider:SetMin(0);
    self.greenSlider:SetMax(255);
    tt(370, 19, self.greenSlider.label, L["Adjust the green value of the selected color."]);
    self.greenSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.g = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end

    line = holder(self, 50);
    self.blueSlider = PengorosPlugins.UI.Slider();
    self.blueSlider:SetParent(line);
    self.blueSlider:SetText(L["Blue"]);
    self.blueSlider:SetPosition(49, 0);
    self.blueSlider:SetSize(350, 40);
    self.blueSlider:SetStep(1);
    self.blueSlider:SetMin(0);
    self.blueSlider:SetMax(255);
    tt(355, 19, self.blueSlider.label, L["Adjust the blue value of the selected color."]);
    self.blueSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.b = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end

    line = holder(self, 50);
    self.alphaSlider = PengorosPlugins.UI.Slider();
    self.alphaSlider:SetParent(line);
    self.alphaSlider:SetText(L["Alpha"]);
    self.alphaSlider:SetPosition(49, 0);
    self.alphaSlider:SetSize(350, 40);
    self.alphaSlider:SetStep(1);
    self.alphaSlider:SetMin(0);
    self.alphaSlider:SetMax(255);
    tt(480, 38, self.alphaSlider.label, L["Adjust the alpha value of the selected color.  0 is fully transparent and 255 is fully opaque."]);
    self.alphaSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.a = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end
end

function OptionsSliderPanel:AddNewSlider()
    local slider = SliderSettings();
    slider.name = L["New Slider"];
    self.sliderCombo:AddItem(slider.name, slider);
    self.sliderCombo:SetSelection(slider);
    self:EnableControls(true);
end

function OptionsSliderPanel:CopyCurrentSlider()
    self:SaveSliderOptions(self.current); -- make sure the options are up to date 
    local slider = self.current:Copy();
    slider.name = slider.name .. " - " .. L["Copy"];
    self.sliderCombo:AddItem(slider.name, slider);
    self.sliderCombo:SetSelection(slider);
    self:EnableControls(true);
end

function OptionsSliderPanel:RemoveCurrentSlider()
    self.sliderCombo:RemoveItem(self.current);
    self:EnableControls(self.sliderCombo:GetItemCount() > 0);
end

function OptionsSliderPanel:ChangeColor(color)
    self.color:SetBackColor(Turbine.UI.Color(color["a"] / 255, color["r"] / 255, color["g"] / 255, color["b"] / 255));
    self.redSlider:SetValue(color["r"]);
    self.greenSlider:SetValue(color["g"]);
    self.blueSlider:SetValue(color["b"]);
    self.alphaSlider:SetValue(color["a"]);
end

function OptionsSliderPanel:Closed()
    self.ignoreWindow:Close();
    self.overrideWindow:Close();
    PengorosPlugins.Utils.KeyManager.RemoveListener(self);
end

function OptionsSliderPanel:UiHidden(hidden)
    if (hidden) then
        self.ignoreWasVisible = self.ignoreWindow:IsVisible();
        self.ignoreWindow:SetVisible(false);
        self.overrideWasVisible = self.overrideWindow:IsVisible();
        self.overrideWindow:SetVisible(false);
    else
        self.ignoreWindow:SetVisible(self.ignoreWasVisible);
        self.overrideWindow:SetVisible(self.overrideWasVisible);
    end
end

function OptionsSliderPanel:EnableControls(enable)
    self.sliderCombo:SetEnabled(enable);
    self.removeButton:SetEnabled(enable);
    self.copyButton:SetEnabled(enable);
    self.nameLabel:SetEnabled(enable);
    self.nameText:SetEnabled(enable);
    self.showSliderCB:SetEnabled(enable);
    self.showDebuffsCB:SetEnabled(enable);
    self.showBuffsCB:SetEnabled(enable);
    --self.showCoolDownsCB:SetEnabled(enable);
    self.visibleTypeLabel:SetEnabled(enable);
    self.visibleTypeCombo:SetEnabled(enable);
    self.timeMaxSlider:SetEnabled(enable);
    self.timeCompressionSlider:SetEnabled(enable);
    self.showCurableOnlyCB:SetEnabled(enable);
    self.showDiseaseCB:SetEnabled(enable);
    self.showFearCB:SetEnabled(enable);
    self.showPoisonCB:SetEnabled(enable);
    self.showWoundCB:SetEnabled(enable);
    self.showCorruptionCB:SetEnabled(enable);
    self.showCryCB:SetEnabled(enable);
    self.showElementalCB:SetEnabled(enable);
    self.showPhysicalCB:SetEnabled(enable);
    self.showSongCB:SetEnabled(enable);
    self.showUndefinedCB:SetEnabled(enable);
    self.minDurationSlider:SetEnabled(enable);
    self.maxDurationSlider:SetEnabled(enable);
--[[
    self.showDiseaseCoolDownCB:SetEnabled(enable);
    self.showFearCoolDownCB:SetEnabled(enable);
    self.showPoisonCoolDownCB:SetEnabled(enable);
    self.showWoundCoolDownCB:SetEnabled(enable);
    self.showMoraleCoolDownCB:SetEnabled(enable);
    self.showPowerCoolDownCB:SetEnabled(enable);
--]]
    self.ignoreButton:SetEnabled(enable);
    self.overrideButton:SetEnabled(enable);
    self.showEffectTimerCB:SetEnabled(enable);
    self.fontLabel:SetEnabled(enable);
    self.fontCombo:SetEnabled(enable);
    self.orientationLabel:SetEnabled(enable);
    self.orientationCombo:SetEnabled(enable);
    self.widthSlider:SetEnabled(enable);
    self.heightSlider:SetEnabled(enable);
    self.colorCombo:SetEnabled(enable);
    self.redSlider:SetEnabled(enable);
    self.greenSlider:SetEnabled(enable);
    self.blueSlider:SetEnabled(enable);
    self.alphaSlider:SetEnabled(enable);
    self.color:SetVisible(enable);
end

function OptionsSliderPanel:SetOptions()
    -- sliders
    local i;
    local size = table.getn(Settings.instance.sliders);
    for i = 1, size do
        local slider = Settings.instance.sliders[i]:Copy();
        self.sliderCombo:AddItem(slider.name, slider);
    end
    
    if (size > 0) then
        self.sliderCombo:SetSelection(self.sliderCombo:GetItem(1));
    else
        self:EnableControls(false);
    end
end

function OptionsSliderPanel:SaveOptions()
    -- save the current slider settings
    if (self.current ~= nil) then
        self:SaveSliderOptions(self.current);
    end
    
    -- collect all the sliders into an array
    local sliders = {};
    local i;
    for i = 1, self.sliderCombo:GetItemCount() do
        sliders[i] = self.sliderCombo:GetItem(i);
    end
    
    -- overwrite the existing sliders
    Settings.instance.sliders = sliders;
end

function OptionsSliderPanel:SetSliderOptions(settings)
    -- ignore 
    self.ignoreWindow:SetOptions(settings);
    self.overrideWindow:SetOptions(settings);
    
    -- general
    self.nameText:SetText(settings.name);
    self.showSliderCB:SetChecked(settings.enabled);
    self.showDebuffsCB:SetChecked(settings.showDebuffs);
    self.showBuffsCB:SetChecked(settings.showBuffs);
    --self.showCoolDownsCB:SetChecked(settings.showCoolDowns);
    self.visibleTypeCombo:SetSelection(settings.visibleType);
    self.timeMaxSlider:SetValue(settings.timeMax);
    self.timeCompressionSlider:SetValue(settings.timeCompression);

    -- effect filters
    self.showCurableOnlyCB:SetChecked(settings.showCurableOnly);
    self.showDiseaseCB:SetChecked(settings.showDisease);
    self.showFearCB:SetChecked(settings.showFear);
    self.showWoundCB:SetChecked(settings.showWound);
    self.showPoisonCB:SetChecked(settings.showPoison);
    self.showCorruptionCB:SetChecked(settings.showCorruption);
    self.showCryCB:SetChecked(settings.showCry);
    self.showElementalCB:SetChecked(settings.showElemental);
    self.showPhysicalCB:SetChecked(settings.showPhysical);
    self.showSongCB:SetChecked(settings.showSong);
    self.showUndefinedCB:SetChecked(settings.showUndefined);
    self.minDurationSlider:SetValue(settings.minDuration);
    self.maxDurationSlider:SetValue(settings.maxDuration);
    
    -- cool down filters
--[[    
    self.showDiseaseCoolDownCB:SetChecked(settings.showDiseaseCD);
    self.showFearCoolDownCB:SetChecked(settings.showFearCD);
    self.showPoisonCoolDownCB:SetChecked(settings.showPoisonCD);
    self.showWoundCoolDownCB:SetChecked(settings.showWoundCD);
    self.showMoraleCoolDownCB:SetChecked(settings.showMoraleCD);
    self.showPowerCoolDownCB:SetChecked(settings.showPowerCD);
--]]
    
    -- ui elements
    self.showEffectTimerCB:SetChecked(settings.showEffectTimer);
    self.formatCombo:SetSelection(settings.displayFormat);
    self.fontCombo:SetSelection(settings.font);
    self.orientationCombo:SetSelection(settings.orientation);
    self.widthSlider:SetValue(settings.barWidth);
    self.heightSlider:SetValue(settings.barHeight);
    
    -- colors
    self.colors = {};
    self.colors[OptionsSliderPanel.Border] = settings.borderColor;
    self.colors[OptionsSliderPanel.Background] = settings.backgroundColor;
    self.colors[OptionsSliderPanel.Font] = settings.fontColor;
    self.colors[OptionsSliderPanel.Outline] = settings.fontOutlineColor;
    self.colorCombo:SetSelection(OptionsSliderPanel.Border);
    self:ChangeColor(self.colors[OptionsSliderPanel.Border]);
end

function OptionsSliderPanel:SaveSliderOptions(settings)
    -- ignore
    self.ignoreWindow:SaveOptions(settings);
    self.overrideWindow:SaveOptions(settings);

    -- general
    settings.name = self.nameText:GetText();
    settings.enabled = self.showSliderCB:IsChecked();
    settings.showDebuffs = self.showDebuffsCB:IsChecked();
    settings.showBuffs = self.showBuffsCB:IsChecked();
    --settings.showCoolDowns = self.showCoolDownsCB:IsChecked();
    settings.visibleType = self.visibleTypeCombo:GetSelection();
    settings.timeMax = self.timeMaxSlider:GetValue();
    settings.timeCompression = self.timeCompressionSlider:GetValue();
   
    -- effect filters
    settings.showCurableOnly = self.showCurableOnlyCB:IsChecked();
    settings.showDisease = self.showDiseaseCB:IsChecked();
    settings.showFear = self.showFearCB:IsChecked();
    settings.showWound = self.showWoundCB:IsChecked();
    settings.showPoison = self.showPoisonCB:IsChecked();
    settings.showCorruption = self.showCorruptionCB:IsChecked();
    settings.showCry = self.showCryCB:IsChecked();
    settings.showElemental = self.showElementalCB:IsChecked();
    settings.showPhysical = self.showPhysicalCB:IsChecked();
    settings.showSong = self.showSongCB:IsChecked();
    settings.showUndefined = self.showUndefinedCB:IsChecked();
    settings.minDuration = self.minDurationSlider:GetValue();
    settings.maxDuration = self.maxDurationSlider:GetValue();
    
    -- cool down filters
--[[
    settings.showDiseaseCD = self.showDiseaseCoolDownCB:IsChecked();
    settings.showFearCD = self.showFearCoolDownCB:IsChecked();
    settings.showPoisonCD = self.showPoisonCoolDownCB:IsChecked();
    settings.showWoundCD = self.showWoundCoolDownCB:IsChecked();
    settings.showMoraleCD = self.showMoraleCoolDownCB:IsChecked();
    settings.showPowerCD = self.showPowerCoolDownCB:IsChecked();
--]]
       
    -- ui elements
    settings.showEffectTimer = self.showEffectTimerCB:IsChecked();
    settings.displayFormat = self.formatCombo:GetSelection();
    settings.font = self.fontCombo:GetSelection();
    settings.orientation = self.orientationCombo:GetSelection();
    settings.barWidth = self.widthSlider:GetValue();
    settings.barHeight = self.heightSlider:GetValue();
   
    -- colors
    settings.borderColor = self.colors[OptionsSliderPanel.Border];
    settings.backgroundColor = self.colors[OptionsSliderPanel.Background];
    settings.fontColor = self.colors[OptionsSliderPanel.Font];
    settings.fontOutlineColor = self.colors[OptionsSliderPanel.Outline];
end
