import "Turbine.Gameplay";
import "Turbine.UI";
import "PengorosPlugins.BuffBars";
import "PengorosPlugins.UI";

--OptionsWindow = class(Turbine.UI.Lotro.Window);
OptionsWindow = class(PengorosPlugins.UI.Window);

function OptionsWindow:Constructor()
    --Turbine.UI.Lotro.Window.Constructor(self);
    PengorosPlugins.UI.Window.Constructor(self);
    
    self:SetText(L["BuffBars Options"]);
    self:SetSize(575, 535);
    self.Closed = function(sender, args)
        PengorosPlugins.UI.ComboBox.Cleanup();
        local i;
        for i = 1, table.getn(self.tabs) do
            if (type(self.tabs[i].Closed) == "function") then
                self.tabs[i]:Closed();
            end
        end
        
        -- save the position
        Settings.instance.optionsX, Settings.instance.optionsY = self:GetPosition();
        Settings.Save();
        
        -- remove the ref
        self:Destroy();
    end
    
    -- restore the position if there is one
    if (Settings.instance.optionsX ~= nil) then
        self:SetPosition(PengorosPlugins.Utils.ValidatePosition(Settings.instance.optionsX, Settings.instance.optionsY, self:GetWidth(), self:GetHeight()));
    end
    
    -- close any open drop downs
    self.PositionChanged = function(sender, args)
        PengorosPlugins.UI.ComboBox.Cleanup();
    end
    
    -- listen for events
    PengorosPlugins.Utils.KeyManager.AddListener(self);

    self.sideBar = Turbine.UI.Control();
    self.sideBar:SetParent(self);
    self.sideBar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.sideBar:SetBackground("PengorosPlugins/UI/Resources/options_menu_side_bar.tga");
    self.sideBar:SetSize(140, 507);
    self.sideBar:SetPosition(432, 24);
    self.sideBar:SetMouseVisible(false);
    
    self.cancelButton = Turbine.UI.Lotro.Button();
    self.cancelButton:SetParent(self);
    self.cancelButton:SetPosition(122, 502);
    self.cancelButton:SetSize(100, 20);
    self.cancelButton:SetText(L["Cancel"]);
    self.cancelButton.Click = function(sender, args)
        self:Close();
    end

    self.acceptButton  = Turbine.UI.Lotro.Button();
    self.acceptButton:SetParent(self);
    self.acceptButton:SetPosition(237, 502);
    self.acceptButton:SetSize(100, 20);
    self.acceptButton:SetText(L["Accept"]);
    self.acceptButton.Click = function(sender, args)
        self:SaveOptions();
        self:Close();
        PengorosPlugins.BuffBars.ResetState();
        Settings.Save();
    end
    
    -- create the tabs
    self.tabs = {OptionsQuickslotPanel(), OptionsEffectPanel(), OptionsSliderPanel(), OptionsLogPanel()};
    self.buttons = {L["Quickslot"], L["Effect Windows"], L["Effect Sliders"], L["Logging"]};
    local i;
    for i = 1, table.getn(self.tabs) do
        -- create a tab button
        local button = PengorosPlugins.UI.Button();
        button:SetParent(self);
        button:SetPosition(439, 60 + (25 * i));
        button:SetSize(127, 20);
        button:SetText(self.buttons[i]);
        button.Click = function(sender, args)
            self:SwitchTab(self.tabs[i], self.buttons[i]);
        end
        self.buttons[i] = button;
        
        -- create the tab
        local tab = Turbine.UI.Control();
        tab:SetParent(self);
        tab:SetSize(415, 447);
        tab:SetPosition(16, 53);
        tab:SetVisible(false);
        
        local scrollbar = Turbine.UI.Lotro.ScrollBar();
        scrollbar:SetOrientation(Turbine.UI.Orientation.Vertical);
        scrollbar:SetParent(tab);
        scrollbar:SetSize(10, 447);
        scrollbar:SetPosition(405, 0);
    
        local panel = self.tabs[i];
        panel.tab = tab;
        panel:SetParent(tab);
        panel:SetVerticalScrollBar(scrollbar);
        panel:SetSize(400, 447);
        panel:SetPosition(0, 0);
        panel:GetItem(1).PositionChanged = function(sender, args)
            PengorosPlugins.UI.ComboBox.Cleanup();
        end
    end

    -- load the options
    self:SetOptions();

    -- show the first tab
    self:SwitchTab(self.tabs[1], self.buttons[1]);
    
    -- make the window visible
    self:SetVisible(true);
end

function OptionsWindow:SwitchTab(tab, button)
    PengorosPlugins.UI.ComboBox.Cleanup();
    
    local i;
    for i = 1, table.getn(self.tabs) do
        self.tabs[i].tab:SetVisible(tab == self.tabs[i]);
        self.buttons[i]:SetSelected(button == self.buttons[i]);
    end
end

function OptionsWindow:UiHidden(hidden)
    self:SetVisible(not hidden);
    PengorosPlugins.UI.ComboBox.Cleanup();
end

function OptionsWindow:SetOptions()
    local i;
    for i = 1, table.getn(self.tabs) do
        self.tabs[i]:SetOptions();
    end
end

function OptionsWindow:SaveOptions()
    local i;
    for i = 1, table.getn(self.tabs) do
        self.tabs[i]:SaveOptions();
    end
end

function OptionsWindow:Destroy()
    self:SetVisible(false);
    PengorosPlugins.Utils.KeyManager.RemoveListener(self);
    local i;
    for i = 1, table.getn(self.tabs) do
        if (type(self.tabs[i].Destroy) == "function") then
            self.tabs[i]:Destroy();
        end
    end
    OptionsWindow.instance = nil;
end

-- class level stuff
OptionsWindow.instance = nil;

-- command to show the options
function OptionsWindow.ShowOptions()
    if (OptionsWindow.instance == nil) then
        OptionsWindow.instance = OptionsWindow();
    end    
end

-- command to close/cleanup the options
function OptionsWindow.CloseOptions()
    if (OptionsWindow.instance ~= nil) then
        OptionsWindow.instance:Destroy();
    end    
end

