import "Turbine.Gameplay";
import "Turbine.UI";
import "PengorosPlugins.BuffBars";

QuickslotBar = class(Turbine.UI.Window);

QuickslotBar.Morale = "morale";
QuickslotBar.Power = "power";

function QuickslotBar:Constructor(settings)
    Turbine.UI.Window.Constructor(self);

    self.settings = settings;
    self.locked = PengorosPlugins.Utils.KeyManager.IsLocked();
    self.hidden = PengorosPlugins.Utils.KeyManager.IsHidden();
    self.quickslots = {};
    self.activeSlots = {};
    self.activeCounts = {};
    
    -- create the quickslots
    local category = Turbine.Gameplay.EffectCategory.Disease;
    self.quickslots[category] = Turbine.UI.Lotro.Quickslot();
    self.quickslots[category]:SetParent(self);
    self.quickslots[category]:SetPosition(0, 0);
    self.quickslots[category]:SetSize(self.settings.quickslotSize, self.settings.quickslotSize);
    
    category = Turbine.Gameplay.EffectCategory.Fear;
    self.quickslots[category] = Turbine.UI.Lotro.Quickslot();
    self.quickslots[category]:SetParent(self);
    self.quickslots[category]:SetPosition(0, 0);
    self.quickslots[category]:SetSize(self.settings.quickslotSize, self.settings.quickslotSize);

    category = Turbine.Gameplay.EffectCategory.Poison;
    self.quickslots[category] = Turbine.UI.Lotro.Quickslot();
    self.quickslots[category]:SetParent(self);
    self.quickslots[category]:SetPosition(0, 0);
    self.quickslots[category]:SetSize(self.settings.quickslotSize, self.settings.quickslotSize);

    category = Turbine.Gameplay.EffectCategory.Wound;
    self.quickslots[category] = Turbine.UI.Lotro.Quickslot();
    self.quickslots[category]:SetParent(self);
    self.quickslots[category]:SetPosition(0, 0);
    self.quickslots[category]:SetSize(self.settings.quickslotSize, self.settings.quickslotSize);

    category = QuickslotBar.Morale;
    self.quickslots[category] = Turbine.UI.Lotro.Quickslot();
    self.quickslots[category]:SetParent(self);
    self.quickslots[category]:SetPosition(0, 0);
    self.quickslots[category]:SetSize(self.settings.quickslotSize, self.settings.quickslotSize);

    category = QuickslotBar.Power;
    self.quickslots[category] = Turbine.UI.Lotro.Quickslot();
    self.quickslots[category]:SetParent(self);
    self.quickslots[category]:SetPosition(0, 0);
    self.quickslots[category]:SetSize(self.settings.quickslotSize, self.settings.quickslotSize);

    -- add liseners
    PengorosPlugins.Utils.EffectManager.AddListener(self);
    PengorosPlugins.Utils.PlayerManager.AddListener(self);
    PengorosPlugins.Utils.KeyManager.AddListener(self);
    
    -- movement listeners
    self.DragEnd = function(sender)
        local left, top = self:GetPosition();
        local width = self:GetWidth();
        self.settings.qsX = left + (width / 2);
        self.settings.qsY = top;
        Settings.Save();
    end
    
    -- check for the window being offscreen
    local width = 6 * self.settings.quickslotSize + 5 * self.settings.quickslotSpacing;
    local x = self.settings.qsX - (width / 2)
    local y = self.settings.qsY;
    x, y = PengorosPlugins.Utils.ValidatePosition(x, y, width, self.settings.quickslotSize);    
    self.settings.qsX = x + (width / 2);
    self.settings.qsY = y;

    -- setup the initial state
    self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));
    self:ResetBar();
    if (not self.locked) then
        self:UiLocked(self.locked);
    end
end

function QuickslotBar:Destroy()
    self:SetVisible(false);
    
    if (self.dragBar ~= nil) then
        self.dragBar:SetVisible(false);
        self.dragBar:SetTarget(nil);
        self.dragBar = nil;
    end
    
    PengorosPlugins.Utils.EffectManager.RemoveListener(self);
    PengorosPlugins.Utils.PlayerManager.RemoveListener(self);
    PengorosPlugins.Utils.KeyManager.RemoveListener(self);
    
    -- if we don't do this on plugin unload the client crashes
    self.quickslots[Turbine.Gameplay.EffectCategory.Disease]:SetShortcut(nil);
    self.quickslots[Turbine.Gameplay.EffectCategory.Fear]:SetShortcut(nil);
    self.quickslots[Turbine.Gameplay.EffectCategory.Poison]:SetShortcut(nil);
    self.quickslots[Turbine.Gameplay.EffectCategory.Wound]:SetShortcut(nil);
    self.quickslots[QuickslotBar.Morale]:SetShortcut(nil);
    self.quickslots[QuickslotBar.Power]:SetShortcut(nil);
end

function QuickslotBar:UpdateVisibility()
    local visible;
    if (self.hidden) then
        visible = false;
    elseif (not self.locked) then
        visible = true;
    else
        visible = table.getn(self.activeSlots) > 0;
        if (visible) then
            if (self.settings.visibleType == WindowVisibility.InCombat) then
                visible = PengorosPlugins.Utils.PlayerManager.GetPlayer():IsInCombat();
            elseif (self.settings.visibleType == WindowVisibility.OutOfCombat) then
                visible = not PengorosPlugins.Utils.PlayerManager.GetPlayer():IsInCombat();
            else -- self.settings.visibleType == WindowVisibility.Always
                visible = true;
            end
        end
    end
    self:SetVisible(visible);
end

function QuickslotBar:InCombatChanged(inCombat)
    self:UpdateVisibility();
end

function QuickslotBar:UiLocked(locked)
    self.locked = locked;
    if (not self.locked) then
        self:EffectsCleared();
        
        -- add all the categories
        if (self.settings.showDisease) then
            self:AddCategory(Turbine.Gameplay.EffectCategory.Disease);
        end
        if (self.settings.showFear) then
            self:AddCategory(Turbine.Gameplay.EffectCategory.Fear);
        end
        if (self.settings.showPoison) then
            self:AddCategory(Turbine.Gameplay.EffectCategory.Poison);
        end
        if (self.settings.showWound) then
            self:AddCategory(Turbine.Gameplay.EffectCategory.Wound);
        end
        if (self.settings.showMorale) then
            self:AddCategory(QuickslotBar.Morale);
        end
        if (self.settings.showPower) then
            self:AddCategory(QuickslotBar.Power);
        end
        
        self.dragBar = PengorosPlugins.UI.DragBar(self, "BuffBars: "..self.settings.name);
        self.dragBar:SetBarOnTop(not self.settings.fillBottomToTop);
        self.dragBar:SetBarVisible(not self.hidden);
        self.dragBar:SetDraggable(true);
        
        self:UpdateVisibility();
    else
        if (self.dragBar ~= nil) then
            self.dragBar:SetDraggable(false);
            self.dragBar:SetTarget(nil);
            self.dragBar = nil;
        end
        
        self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));
        self:ResetBar();
    end
end

function QuickslotBar:UiHidden(hidden)
    self.hidden = hidden;
    if (self.dragBar ~= nil) then
        self.dragBar:SetVisible(not hidden);
    end
    self:UpdateVisibility();
end

function QuickslotBar:ResetBar()
    -- clear the exsiting categories
    self.activeSlots = {};
    local category = Turbine.Gameplay.EffectCategory.Disease;
    self.quickslots[category]:SetVisible(false);
    self.activeCounts[category] = 0;
    pcall(function()
        self.quickslots[category]:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.diseaseType, self.settings.diseaseData));
    end);
    
    category = Turbine.Gameplay.EffectCategory.Fear;
    self.quickslots[category]:SetVisible(false);
    self.activeCounts[category] = 0;
    pcall(function()
        self.quickslots[category]:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.fearType, self.settings.fearData));
    end);

    category = Turbine.Gameplay.EffectCategory.Poison;
    self.quickslots[category]:SetVisible(false);
    self.activeCounts[category] = 0;
    pcall(function()
        self.quickslots[category]:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.poisonType, self.settings.poisonData));
    end);

    category = Turbine.Gameplay.EffectCategory.Wound;
    self.quickslots[category]:SetVisible(false);
    self.activeCounts[category] = 0;
    pcall(function()
        self.quickslots[category]:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.woundType, self.settings.woundData));
    end);

    category = QuickslotBar.Morale;
    self.quickslots[category]:SetVisible(false);
    self.activeCounts[category] = 0;
    pcall(function()
        self.quickslots[category]:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.moraleType, self.settings.moraleData));
    end);

    category = QuickslotBar.Power;
    self.quickslots[category]:SetVisible(false);
    self.activeCounts[category] = 0;
    pcall(function()
        self.quickslots[category]:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.powerType, self.settings.powerData));
    end);

    -- add all the existing effects
    local i;
    local effects = PengorosPlugins.Utils.EffectManager.GetEffects();
    for i = 1, effects:GetCount() do
        self:EffectAdded(effects:Get(i));
    end
        
    -- check existing morale/power levels
    self:MoraleChanged(PengorosPlugins.Utils.PlayerManager.GetPlayer());
    self:PowerChanged(PengorosPlugins.Utils.PlayerManager.GetPlayer());

    -- force a layout
    self:Layout();
end

function QuickslotBar:EffectsCleared()
    for category,count in pairs(self.activeCounts) do
        if (count > 0) then
            self.activeCounts[category] = 1;
            self:RemoveCategory(category);
        end
    end
end

function QuickslotBar:EffectAdded(effect)
    -- don't add pots in unlocked mode
    if (not self.locked) then
        return;
    end
    
    -- is the effect filtered
    if (self.settings:IsEffectFiltered(effect)) then
        return;
    end
    
    self:AddCategory(effect:GetCategory());
end

function QuickslotBar:EffectRemoved(effect)
    -- don't add pots in unlocked mode
    if (not self.locked) then
        return;
    end
    
    -- is the effect filtered
    if (self.settings:IsEffectFiltered(effect)) then
        return;
    end
    
    self:RemoveCategory(effect:GetCategory());
end

function QuickslotBar:AddCategory(category)
    -- update the slot counts
    self.activeCounts[category] = self.activeCounts[category] + 1;

    -- if the count is one we need to set the slot visible and relayout the widget
    if (self.activeCounts[category] == 1) then
        self.quickslots[category]:SetVisible(true);

        table.insert(self.activeSlots, self.quickslots[category]);
        self:Layout();
    end
end

function QuickslotBar:RemoveCategory(category)
    -- update the slot counts
    self.activeCounts[category] = self.activeCounts[category] - 1;

    -- if the count is 0 we need to get rid of the slot and relayout
    if (self.activeCounts[category] == 0) then
        local size = table.getn(self.activeSlots);
        for i = 1, size, 1 do
            if (self.activeSlots[i] == self.quickslots[category]) then
                table.remove(self.activeSlots, i);
                break;
            end
        end

        -- hide the quickslot
        self.quickslots[category]:SetVisible(false);

        -- fix the layout
        self:Layout();        
    end
end

function QuickslotBar:MoraleChanged(player)
    if (self.locked == false) then return;
    end
    
    if (not self.settings.showMorale) then
        return;
    end

    local maxMorale = player:GetMaxMorale();
    local morale = player:GetMorale();
    local percent = morale / maxMorale;
    local category = QuickslotBar.Morale;
    
    -- see if we need to show it
    if (percent <= self.settings.moralePercent) then
        if (self.activeCounts[category] == 0) then
            self:AddCategory(category);
        end
    else
        if (self.activeCounts[category] == 1) then
            self:RemoveCategory(category);
        end    
    end
end

function QuickslotBar:PowerChanged(player)
    if (self.locked == false) then return;
    end
    
    if (not self.settings.showPower) then
        return;
    end

    local maxPower = player:GetMaxPower();
    local power = player:GetPower();
    local percent = power / maxPower;
    local category = QuickslotBar.Power;
    
    -- see if we need to show it
    if (percent <= self.settings.powerPercent) then
        if (self.activeCounts[category] == 0) then
            self:AddCategory(category);
        end
    else
        if (self.activeCounts[category] == 1) then
            self:RemoveCategory(category);
        end    
    end
end

function QuickslotBar:Layout()
    local size = table.getn(self.activeSlots);
    for i = 1, size, 1 do
        self.activeSlots[i]:SetPosition((i - 1) * (self.settings.quickslotSize + self.settings.quickslotSpacing), 0);
    end

    if (size > 0) then
        local width = size * self.settings.quickslotSize + (size - 1) * self.settings.quickslotSpacing;
        self:SetSize(width, self.settings.quickslotSize);
        self:SetPosition(self.settings.qsX - (width / 2), self.settings.qsY);
    end
    self:UpdateVisibility();
end
