import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

QuickslotSettings = class();

function QuickslotSettings:Constructor(saved)
    if (saved ~= nil) then
        for i,v in pairs(saved) do
            self[i] = v;
        end
    else
        self:SetDefaults();
    end
end

function QuickslotSettings:Copy()
    return PengorosPlugins.Utils.DeepCopy(self);
end

function QuickslotSettings:SetDefaults()
    local displayWidth = Turbine.UI.Display:GetWidth();
    local displayHeight = Turbine.UI.Display:GetHeight();
    
    self.enabled = true;
    self.name = "Quickslots";
    self.visibleType = WindowVisibility.Always; -- v4
    self.quickslotSize = 36;
    self.quickslotSpacing = 5;
    self.qsX = displayWidth / 2;
    self.qsY = ((displayHeight - 300) / 2) + 300;
    self.moralePercent = .30;
    self.powerPercent = .30;
    
    -- filters
    self.showDisease = true;
    self.showFear = true;
    self.showPoison = true;
    self.showWound = true;
    self.showMorale = true;
    self.showPower = true;
end

function QuickslotSettings:IsEffectFiltered(effect)
    local category = effect:GetCategory();
    if (not effect:IsCurable()) then
        return true;
    elseif (not effect:IsDebuff()) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Disease) and (not self.showDisease)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Fear) and (not self.showFear)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Wound) and (not self.showWound)) then
        return true;
    elseif((category == Turbine.Gameplay.EffectCategory.Poison) and (not self.showPoison)) then
        return true;
    end
    
    -- not filtered
    return false;
end
