import "Turbine.UI";
import "Turbine.UI.Lotro";
import "PengorosPlugins.BuffBars";

SliderItem = class(Turbine.UI.Lotro.EffectDisplay);

function SliderItem:Constructor(parent, effect, settings)
    Turbine.UI.Lotro.EffectDisplay.Constructor(self);

    -- vars
    self.settings = settings;
    self.lastRemaining = -1;
    self.startTime = effect:GetStartTime();
    self.duration = effect:GetDuration();
    self.endTime = (self.startTime + self.duration);
    
    -- effect 
    local gameTime = Turbine.Engine.GetGameTime();
    self:SetParent(parent);
    self:SetEffect(effect);
    self:SetSize(self.settings.barHeight, self.settings.barHeight);
    if (gameTime < self.endTime) then
        self:SetWantsUpdates(true);
    end
    
    -- hidden window
    self.window = Turbine.UI.Window();
    self.window:SetParent(parent);
    self.window:SetSize(self.settings.barHeight, self.settings.barHeight);
    
    self:SetPosition(parent:CalculatePosition(self.endTime - gameTime));
    -- label
    if (self.settings.showEffectTimer) then
        self.timer = Turbine.UI.Label();
        self.timer:SetParent(self.window);
        self.timer:SetMouseVisible(false);
        self.timer:SetSize(self.settings.barHeight, self.settings.barHeight);
        self.timer:SetPosition(0, 0);
        self.timer:SetForeColor(self.settings:GetFontColor());
        self.timer:SetOutlineColor(self.settings:GetFontOutlineColor());
        self.timer:SetFontStyle(Turbine.UI.FontStyle.Outline);
        self.timer:SetFont(self.settings.font);
        self.timer:SetTextAlignment(Turbine.UI.ContentAlignment.BottomCenter);
    end
    
    -- add a context menu to show the options
    local contextMenu = Turbine.UI.ContextMenu();
    local menuItems = contextMenu:GetItems();
    menuItems:Add(Turbine.UI.MenuItem(L["Ignore"]));
    menuItems:Add(Turbine.UI.MenuItem(L["Always Show"]));
    menuItems:Add(Turbine.UI.MenuItem(L["Options"]));
    
    menuItems:Get(1).Click = function(sender, args)
        self:AddToIgnore();
    end;
    menuItems:Get(2).Click = function(sender, args)
        self:AddToOverride();
    end;
    menuItems:Get(3).Click = function(sender, args)
        OptionsWindow.ShowOptions();
    end;

    self.window.MouseClick = function(sender, args)
        if (args.Button == Turbine.UI.MouseButton.Right) then
            contextMenu:ShowMenu();
        end
    end
end

function SliderItem:Destroy()
    self:SetWantsUpdates(false);
    self:SetVisible(false);
    self.window:SetVisible(false);
    self.window.MouseClick = nil;
end

function SliderItem:SetPosition(x, y)
    self.window:SetPosition(x, y);
    Turbine.UI.Lotro.EffectDisplay.SetPosition(self, x, y);
end

function SliderItem:AddToIgnore()
    local effect = self:GetEffect();
    -- remove it from the bar and add it to the ignore list
    self.RemoveEffect(effect);
    self.settings:AddIgnoredEffect(effect:GetName());
    Settings.Save();
end

function SliderItem:AddToOverride()
    local effect = self:GetEffect();
    self.settings:AddOverrideEffect(effect:GetName());
    Settings.Save();
end

function SliderItem:Update(args)
    local gameTime = Turbine.Engine.GetGameTime();

    -- update the text if needed
    local remaining = math.max(0, self.endTime - gameTime);
    local iRemaining = math.floor(remaining);
    if (self.timer ~= nil and iRemaining ~= self.lastRemaining) then
        local format;
        if (self.settings.displayFormat == TimeDisplayFormat.Truncated) then
            format = PengorosPlugins.Utils.TimeFormat.TRUNCATED;
        elseif (self.settings.displayFormat == TimeDisplayFormat.Clock) then
            format = PengorosPlugins.Utils.TimeFormat.CLOCK;
        else
            format = PengorosPlugins.Utils.TimeFormat.LITERAL;
        end
        self.timer:SetText(format(iRemaining));
        self.lastRemaining = iRemaining;
    end
    
    -- set the new position
    self:SetPosition(self.window:GetParent():CalculatePosition(remaining));
    
    if (gameTime > self.endTime) then
        self:SetWantsUpdates(false);
    end        
end
